@@ -1,6 +1,6 @@
 # check_perl_modules.pl - This script checks installed and required Perl modules
 # It also does the required installations
-# RCS Identication ; $Revision: 1.27.2.1 $ ; $Date: 2004/04/19 13:07:29 $ 
+# RCS Identication ; $Revision: 1.47.2.1 $ ; $Date: 2006/06/08 12:53:26 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -43,7 +43,9 @@ use CPAN;
 	     'Net::SSLeay' => '1.16',
 	     'Archive::Zip' => '1.05',
 	     'Bundle::LWP' => '1.09',
-	     'SOAP::Lite' => '0.60'
+	     'SOAP::Lite' => '0.60',
+	     'MHonArc::UTF8' => '2.6.0',
+	     'MIME::Base64' => '3.03'
 	     );
 
 ### key:left "module" used by SYMPA, 
@@ -56,19 +58,27 @@ use CPAN;
 	     'MIME::Base64' => 'MIME-Base64',
 	     'CGI' => 'CGI',
 	     'File::Spec' => 'File-Spec',
+	     'Regexp::Common' => 'Regexp-Common',
+	     'Locale::TextDomain' => 'libintl-perl',
+	     'Template' => 'Template-Toolkit',
 	     'Archive::Zip' => 'Archive-Zip',
+	     'LWP' => 'libwww-perl',
+             'XML::LibXML' => 'XML-LibXML',
+	     'MHonArc::UTF8' => 'MHonArc',
+	     'FCGI' => 'FCGI',
+	     'DBI' => 'DBI',
+	     'DBD::mysql' => 'Msql-Mysql-modules',
+	     'Crypt::CipherSaber' => 'CipherSaber',
+	     'Encode' => 'Encode',
 	     );
 
-%opt_CPAN = ('DBI' => 'DBI',
-	     'DBD::mysql' => 'Msql-Mysql-modules',
-	     'DBD::Pg' => 'DBD-Pg',
+%opt_CPAN = ('DBD::Pg' => 'DBD-Pg',
 	     'DBD::Oracle' => 'DBD-Oracle',
 	     'DBD::Sybase' => 'DBD-Sybase',
+	     'DBD::SQLite' => 'DBD-SQLite',
 	     'Net::LDAP' =>   'perl-ldap',
 	     'CGI::Fast' => 'CGI',
-             'Crypt::CipherSaber' => 'CipherSaber',
-	     'FCGI' => 'FCGI',
-	     'Net::SMTP' => 'libnet',
+ 	     'Net::SMTP' => 'libnet',
 	     'IO::Socket::SSL' => 'IO-Socket-SSL',
 	     'Net::SSLeay' => 'NET-SSLeay',
 	     'Bundle::LWP' => 'LWP',
@@ -79,6 +89,7 @@ use CPAN;
 		 'DBD::Pg' => 'PostgreSQL database driver, required if you connect to a PostgreSQL database.',
 		 'DBD::Oracle' => 'Oracle database driver, required if you connect to a Oracle database.',
 		 'DBD::Sybase' => 'Sybase database driver, required if you connect to a Sybase database.',
+		 'DBD::SQLite' => 'Sybase database driver, required if you connect to a SQLite database.',
 		 'Net::LDAP' =>   'required to query LDAP directories. Sympa can do LDAP-based authentication ; it can also build mailing lists with LDAP-extracted members.',
 		 'CGI::Fast' => 'WWSympa, Sympa\'s web interface can run as a FastCGI (ie: a persistent CGI). If you install this module, you will also need to install the associated mod_fastcgi for Apache.',
 		 'Crypt::CipherSaber' => 'this module provides reversible encryption of user passwords in the database.',
@@ -123,9 +134,13 @@ sub check_modules {
 
     foreach $mod (sort keys %todo) {
 	printf ("%-20s %-15s", $mod, $todo{$mod});
+	
 	$status = &test_module($mod);
 	if ($status == 1) {
 	    $vs = "$mod" . "::VERSION";
+
+	    $vs = 'mhonarc::VERSION' if $mod =~ /^mhonarc/i;
+
 	    $v = $$vs;
 	    $rv = $versions{$mod} || "1.0" ;
 	    ### OK: check version
@@ -165,7 +180,7 @@ sub install_module {
 
     ## This is required on RedHat 9 for DBD::mysql installation
     my $lang = $ENV{'LANG'};
-    $ENV{'LANG'} = 'C' if ($ENV{'LANG'} eq 'en_US.UTF-8');
+    $ENV{'LANG'} = 'C' if ($ENV{'LANG'} =~ /UTF\-8/);
 
     unless ($> == 0) {
 	print "\#\# You need root privileges to install $module module. \#\#\n";
@@ -197,6 +212,10 @@ sub test_module {
 
     $filename =~ s/::/\//g;
     $filename .= ".pm";
+    
+    ## Exception for mhonarc
+    $filename = 'mhamain.pl' if $filename =~ /^mhonarc/i;
+
     return 1 if $INC{$filename};
     
   ITER: {
