@@ -1,5 +1,5 @@
 # parse_tex.pl - This script parses parts of the sympa.tex.tpl
-# RCS Identication ; $Revision: 1.7.2.1 $ ; $Date: 2003/03/17 08:33:45 $ 
+# RCS Identication ; $Revision: 1.9 $ ; $Date: 2003/12/22 13:22:18 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -38,7 +38,6 @@ my $version = <VERSION>;
 chomp $version;
 close VERSION;
 
-
 ## Init struct
 my %data = ('escaped_start' => '[STARTPARSE]',
 	    'escaped_stop' => '[STOPPARSE]',
@@ -46,8 +45,11 @@ my %data = ('escaped_start' => '[STARTPARSE]',
 	    'version' => $version
 	    );
 
-foreach my $key (%ENV) {
-    $data{$key} = $ENV{$key};
+## All DIR variables
+foreach my $k (keys %ENV) {
+#    if ($k =~ /DIR$/) {
+	$data{$k} = $ENV{$k};
+#    }
 }
 
 ## scenari
