@@ -34,6 +34,7 @@ use Conf;
 use Log;
 use Auth;
 use CAS;
+use Language;
 
 ## Define types of SOAP type listType
 my %types = ('listType' => {'listAddress' => 'string',
@@ -81,18 +82,21 @@ sub lists {
     
     &do_log('info', 'SOAP lists(%s,%s)', $topic, $subtopic);
    
-    foreach my $listname ( &List::get_lists($robot) ) {
+    my $all_lists = &List::get_lists($robot);
+    foreach my $list ( @$all_lists ) {
 	
-	my $list = new List ($listname, $robot);
+	my $listname = $list->{'name'};
+
 	my $result_item = {};
-	my $action = &List::request_action ('visibility','md5',$robot,
-					    {'listname' =>  $listname,
-					     'sender' => $sender}
+	my $result = $list->check_list_authz('visibility','md5',
+					    {'sender' => $sender}
 					    );
+	my $action;
+	$action = $result->{'action'} if (ref($result) eq 'HASH');
 	next unless ($action eq 'do_it');
 	
 	##building result packet
-	$result_item->{'listAddress'} = $listname.'@'.$list->{'domain'};
+	$result_item->{'listAddress'} = $listname.'@'.$list->{'admin'}{'host'};
 	$result_item->{'subject'} = $list->{'admin'}{'subject'};
 	$result_item->{'subject'} =~ s/;/,/g;
 	$result_item->{'homepage'} = &Conf::get_robot_conf($robot,'wwsympa_url').'/info/'.$listname; 
@@ -131,6 +135,7 @@ sub login {
     my $passwd = shift;
 
     my $http_host = $ENV{'SERVER_NAME'};
+    my $robot = $ENV{'SYMPA_ROBOT'};
     &Log::do_log('notice', 'login(%s)', $email);
     
     #foreach my  $k (keys %ENV) {
@@ -147,7 +152,7 @@ sub login {
     ## Authentication of the sender
     ## Set an env var to find out if in a SOAP context
     $ENV{'SYMPA_SOAP'} = 1;
-    my $user = &Auth::check_auth($email,$passwd);
+    my $user = &Auth::check_auth($robot, $email,$passwd);
 
     unless($user){
 	&do_log('notice', "SOAP : login authentication failed");
@@ -190,8 +195,8 @@ sub casLogin {
     ## Validate the CAS ST against all known CAS servers defined in auth.conf
     ## CAS server response will include the user's NetID
     my ($user, @proxies, $email, $cas_id);
-    foreach my $service_id (0..$#{$Conf{'auth_services'}}){
-	my $auth_service = $Conf{'auth_services'}[$service_id];
+    foreach my $service_id (0..$#{$Conf{'auth_services'}{$robot}}){
+	my $auth_service = $Conf{'auth_services'}{$robot}[$service_id];
 	next unless ($auth_service->{'auth_type'} eq 'cas'); ## skip non CAS entries
 	
 	my $cas = new CAS(casUrl => $auth_service->{'base_url'}, 
@@ -219,7 +224,7 @@ sub casLogin {
     } 
 
     ## Now fetch email attribute from LDAP
-    unless ($email = &Auth::cas_get_email_by_net_id($user, $cas_id)) {
+    unless ($email = &Auth::get_email_by_net_id($robot, $cas_id, {'uid' => $user})) {
 	&do_log('err','Could not get email address from LDAP for user %s', $user);
 	die SOAP::Fault->faultcode('Server')
 	    ->faultstring('Authentification failed')
@@ -338,24 +343,26 @@ sub info {
     # Part of the authorization code
     $user = &List::get_user_db($sender);
      
-    my $action = &List::request_action ('info','md5',$robot,
-                                     {'listname' => $listname,
-                                      'sender' => $sender});
+    my $result = $list->check_list_authz('info','md5',
+					 {'sender' => $sender});
+    my $action;
+    $action = $result->{'action'} if (ref($result) eq 'HASH');
 
     die SOAP::Fault->faultcode('Server')
 	->faultstring('No action available')
 	unless (defined $action);
 
-    if ($action =~ /reject(\(\'?(\w+)\'?\))?/i) {
+    if ($action =~ /reject/i) {
+	my $reason_string = &get_reason_string($result->{'reason'},$robot);
 	&Log::do_log('info', 'SOAP : info %s from %s refused (not allowed)', $listname,$sender);
 	die SOAP::Fault->faultcode('Server')
 	    ->faultstring('Not allowed')
-	    ->faultdetail("You don't have proper rights");
+	    ->faultdetail($reason_string);
     }
     if ($action =~ /do_it/i) {
 	my $result_item;
 
-	$result_item->{'listAddress'} = SOAP::Data->name('listAddress')->type('string')->value($listname.'@'.$list->{'domain'});
+	$result_item->{'listAddress'} = SOAP::Data->name('listAddress')->type('string')->value($listname.'@'.$list->{'admin'}{'host'});
 	$result_item->{'subject'} = SOAP::Data->name('subject')->type('string')->value($list->{'admin'}{'subject'});
 	$result_item->{'homepage'} = SOAP::Data->name('homepage')->type('string')->value(&Conf::get_robot_conf($robot,'wwsympa_url').'/info/'.$listname); 
 	
@@ -418,19 +425,21 @@ sub review {
     # Part of the authorization code
     $user = &List::get_user_db($sender);
      
-    my $action = &List::request_action ('review','md5',$robot,
-                                     {'listname' => $listname,
-                                      'sender' => $sender});
+    my $result = $list->check_list_authz('review','md5',
+					 {'sender' => $sender});
+    my $action;
+    $action = $result->{'action'} if (ref($result) eq 'HASH');
 
     die SOAP::Fault->faultcode('Server')
 	->faultstring('No action available')
 	unless (defined $action);
 
-    if ($action =~ /reject(\(\'?(\w+)\'?\))?/i) {
+    if ($action =~ /reject/i) {
+	my $reason_string = &get_reason_string($result->{'reason'},$robot);
 	&Log::do_log('info', 'SOAP : review %s from %s refused (not allowed)', $listname,$sender);
 	die SOAP::Fault->faultcode('Server')
 	    ->faultstring('Not allowed')
-		->faultdetail("You don't have proper rights");
+		->faultdetail($reason_string);
     }
     if ($action =~ /do_it/i) {
 	my $is_owner = $list->am_i('owner', $sender);
@@ -494,7 +503,9 @@ sub signoff {
     
     if ($listname eq '*') {
 	my $success;
-	foreach $l ( List::get_which ($sender,$robot,'member') ){
+	foreach my $list  ( &List::get_which ($sender,$robot,'member') ){
+	    my $l = $list->{'name'};
+
 	    $success ||= &signoff($l,$sender);
 	}
 	return SOAP::Data->name('result')->value($success);
@@ -505,21 +516,23 @@ sub signoff {
     # Part of the authorization code
     my $user = &List::get_user_db($sender);
     
-    my $action = &List::request_action('unsubscribe','md5',$robot,
-				       {'listname' => $listname,
-					'email' => $sender,
-					'sender' => $sender });
-    
+    my $result = $list->check_list_authz('unsubscribe','md5',
+					 {'email' => $sender,
+					  'sender' => $sender });
+    my $action;
+    $action = $result->{'action'} if (ref($result) eq 'HASH');
+
     die SOAP::Fault->faultcode('Server')
 	->faultstring('No action available.')
 	unless (defined $action);   
     
-    if ($action =~ /reject(\(\'?(\w+)\'?\))?/i) {
+    if ($action =~ /reject/i) {
+	my $reason_string = &get_reason_string($result->{'reason'},$robot);
 	&Log::do_log('info', 'SOAP : sign off from %s for the email %s of the user %s refused (not allowed)', 
 		     $listname,$sender,$sender);
 	die SOAP::Fault->faultcode('Server')
 	    ->faultstring('Not allowed.')
-	    ->faultdetail("You don't have proper rights");
+	    ->faultdetail($reason_string);
     }
     if ($action =~ /do_it/i) {
 	## Now check if we know this email on the list and
@@ -530,8 +543,9 @@ sub signoff {
 	    
 	    ## Tell the owner somebody tried to unsubscribe
 	    if ($action =~ /notify/i) {
-		$list->send_notify_to_owner({'who' => $sender, 
-					     'type' => 'warn-signoff'});
+		unless ($list->send_notify_to_owner('warn-signoff',{'who' => $sender})) {
+		    &Log::do_log('err',"Unable to send notify 'warn-signoff' to $list->{'name'} listowner");
+		}
 	    }
 	    die SOAP::Fault->faultcode('Server')
 		->faultstring('Not allowed.')
@@ -543,15 +557,16 @@ sub signoff {
 
 	## Notify the owner
 	if ($action =~ /notify/i) {
-	    $list->send_notify_to_owner({'who' => $sender,
-					 'type' => 'signoff'});
+	    unless ($list->send_notify_to_owner('notice',{'who' => $sender,
+							  'command' => 'signoff'})) {
+		&Log::do_log('err',"Unable to send notify 'notice' to $list->{'name'} listowner");
+	    }
 	}
 
 	## Send bye.tpl to sender
-	my %context;
-	$context{'subject'} = sprintf(&Language::Msg(6 , 71, 'Signoff from list %s'), $list->{'name'});
-	$context{'body'} = sprintf(&Language::Msg(6 , 31, "You have been removed from list %s.\n Thanks for being with us.\n"), $list->{'name'});
-	$list->send_file('bye', $sender, $robot, \%context);
+	unless ($list->send_file('bye', $sender, $robot,{})) {
+	    &Log::do_log('err',"Unable to send template 'bye' to $sender");
+	}
 	
 	$list->save();
 
@@ -560,7 +575,7 @@ sub signoff {
 	return SOAP::Data->name('result')->type('boolean')->value(1);
     }
 
-  &Log::do_log('info', 'SOAP : sign off %s from %s aborted, unknown requested action in scenario',$listname,$sender);
+    &Log::do_log('info', 'SOAP : sign off %s from %s aborted, unknown requested action in scenario',$listname,$sender);
   die SOAP::Fault->faultcode('Server')
       ->faultstring('Undef')
 	  ->faultdetail("Sign off %s from %s aborted because unknown requested action in scenario",$listname,$sender);
@@ -604,27 +619,36 @@ sub subscribe {
   $gecos = "\"$gecos\"" if ($gecos =~ /[<>\(\)]/);
   
   ## query what to do with this subscribtion request
-  my $action = &List::request_action('subscribe','md5',$robot,
-				     {'listname' => $listname,
-				      'sender' => $sender });
- 
+  my $result = $list->check_list_authz('subscribe','md5',
+				       {'sender' => $sender });
+  my $action;
+  $action = $result->{'action'} if (ref($result) eq 'HASH');
+
   die SOAP::Fault->faultcode('Server')
       ->faultstring('No action available.')
 	  unless (defined $action); 
   
   &Log::do_log('debug2', 'SOAP subscribe action : %s', $action);
   
-  if ($action =~ /reject(\(\'?(\w+)\'?\))?/i) {
+  if ($action =~ /reject/i) {
+      my $reason_string = &get_reason_string($result->{'reason'},$robot);
       &Log::do_log('info', 'SOAP subscribe to %s from %s refused (not allowed)', $listname,$sender);
       die SOAP::Fault->faultcode('Server')
 	  ->faultstring('Not allowed.')
-	  ->faultdetail("You don't have proper rights");
+	  ->faultdetail($reason_string);
   }
   if ($action =~ /owner/i) {
-      push @msg::report, sprintf Msg(6, 25, $msg::subscription_forwarded);
+       
       ## Send a notice to the owners.
       my $keyauth = $list->compute_auth($sender,'add');
-      $list->send_sub_to_owner($sender, $keyauth, &Conf::get_robot_conf($robot, 'sympa'), $gecos);
+      unless ($list->send_notify_to_owner('subrequest',{'who' => $sender,
+				   'keyauth' => $list->compute_auth($sender,'add'),
+				   'replyto' => &Conf::get_robot_conf($robot, 'sympa'),
+							'gecos' => $gecos})) {
+	  &Log::do_log('err',"Unable to send notify 'subrequest' to $list->{'name'} listowner");
+      }
+
+#      $list->send_sub_to_owner($sender, $keyauth, &Conf::get_robot_conf($robot, 'sympa'), $gecos);
       $list->store_susbscription_request($sender, $gecos);
       &Log::do_log('info', 'SOAP subscribe : %s from %s forwarded to the owners of the list (%d seconds)',$listname,$sender,time-$time_command);
       return SOAP::Data->name('result')->type('boolean')->value(1);
@@ -689,17 +713,18 @@ sub subscribe {
 
       ## Now send the welcome file to the user
       unless ($quiet || ($action =~ /quiet/i )) {
-	  my %context;
-	  $context{'subject'} = sprintf(&Language::Msg(8, 6, "Welcome to list %s"), $list->{'name'});
-	  $context{'body'} = sprintf(&Language::Msg(8, 6, "You are now subscriber of list %s"), $list->{'name'});
-	  $list->send_file('welcome', $sender, $robot, \%context);
+	  unless ($list->send_file('welcome', $sender, $robot,{})) {
+	      &Log::do_log('err',"Unable to send template 'bye' to $sender");
+	  }
       }
       
       ## If requested send notification to owners
       if ($action =~ /notify/i) {
-	  $list->send_notify_to_owner({'who' => $sender,
+	  unless ($list->send_notify_to_owner('notice',{'who' => $sender,
 				       'gecos' => $gecos,
-				       'type' => 'subscribe'});
+							'command' => 'subscribe'})) {
+	      &Log::do_log('err',"Unable to send notify 'notice' to $list->{'name'} listowner");
+	  }
       }
       &Log::do_log('info', 'SOAP subcribe : %s from %s accepted', $listname, $sender);
       
@@ -753,19 +778,32 @@ sub which {
 	    ->faultdetail('You should login first');
     }
     
-    foreach my $listname( &List::get_which($sender,$robot, 'member') ){ 	    
-	my $list = new List ($listname, $robot);
+    my %listnames;
+
+    
+    foreach my $role ('member','owner','editor') {
+	foreach my $list( &List::get_which($sender,$robot,$role) ){         
+	    my $name = $list->{'name'};
+	    $listnames{$name} = $list;
+	}
+    }
+    
+    foreach my $name (keys %listnames) {
+	my $list = $listnames{$name};
+
 	my $list_address;
 	my $result_item;
 
-	next unless (&List::request_action ('visibility', 'md5', $robot,
-					    {'listname' =>  $listname,
-					     'sender' =>$sender}) =~ /do_it/);
-	
-	$result_item->{'listAddress'} = $listname.'@'.$list->{'domain'};
+	my $result = $list->check_list_authz('visibility', 'md5',
+					     {'sender' =>$sender});
+	my $action;
+	$action = $result->{'action'} if (ref($result) eq 'HASH');
+	next unless ($action =~ /do_it/i);
+
+	$result_item->{'listAddress'} = $name.'@'.$list->{'admin'}{'host'};
 	$result_item->{'subject'} = $list->{'admin'}{'subject'};
 	$result_item->{'subject'} =~ s/;/,/g;
-	$result_item->{'homepage'} = &Conf::get_robot_conf($robot,'wwsympa_url').'/info/'.$listname;
+	$result_item->{'homepage'} = &Conf::get_robot_conf($robot,'wwsympa_url').'/info/'.$name;
 	 
 	## determine status of user 
 	$result_item->{'isOwner'} = 0;
@@ -827,4 +865,22 @@ sub struct_to_soap {
     return $soap_data;
 }
 
+
+sub get_reason_string {
+    my ($reason,$robot) = @_;
+
+    my $data = {'reason' => $reason };
+    my $string;
+    my $tt2_include_path =  &tools::make_tt2_include_path($robot,'mail_tt2','','');
+
+    unless (&tt2::parse_tt2($data,'authorization.tt2' ,\$string, $tt2_include_path)) {
+	my $error = &tt2::get_error();
+	&List::send_notify_to_listmaster('web_tt2_error', $robot, [$error]);
+	&do_log('info', "get_reason_string : error parsing");
+	return '';
+    }
+    
+    return $string
+}
+
 1;
