@@ -1,5 +1,5 @@
 # Conf.pm - This module does the sympa.conf and robot.conf parsing
-# RCS Identication ; $Revision: 1.83.2.2 $ ; $Date: 2004/08/05 09:18:12 $ 
+# RCS Identication ; $Revision: 1.138.2.2 $ ; $Date: 2006/08/24 15:05:36 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -35,30 +35,37 @@ use Carp;
 @EXPORT = qw(%Conf);
 
 my @valid_options = qw(
-		       avg bounce_warn_rate bounce_halt_rate chk_cert_expiration_task expire_bounce_task
-		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod 
-		       cookie cookie_cas_expire create_list crl_dir crl_update_task db_host db_env db_name 
+		       avg bounce_warn_rate bounce_halt_rate bounce_email_prefix chk_cert_expiration_task expire_bounce_task
+		       cache_list_config
+		       clean_delay_queue clean_delay_queueauth clean_delay_queuemod clean_delay_queuesubscribe clean_delay_queuetopic default_remind_task
+		       cookie cookie_cas_expire create_list crl_dir crl_update_task db_host db_env db_name db_timeout
 		       db_options db_passwd db_type db_user db_port db_additional_subscriber_fields db_additional_user_fields
-		       default_shared_quota default_archive_quota default_list_priority edit_list email etc
-		       global_remind home host domain lang listmaster log_socket_type log_level 
-		       misaddressed_commands misaddressed_commands_regexp max_size maxsmtp msgcat nrcpt 
-		       owner_priority pidfile spool queue queueauth queuetask queuebounce queuedigest 
-		       queueexpire queuemod queuesubscribe queueoutgoing tmpdir
-		       loop_command_max loop_command_sampling_delay loop_command_decrease_factor
+		       default_shared_quota default_archive_quota default_list_priority distribution_mode edit_list email etc
+		       global_remind home host domain lang listmaster listmaster_email localedir log_socket_type log_level 
+		       logo_html_definition misaddressed_commands misaddressed_commands_regexp max_size maxsmtp nrcpt 
+		       owner_priority pidfile pidfile_distribute
+		       spool queue queuedistribute queueauth queuetask queuebounce queuedigest 
+		       queuemod queuetopic queuesubscribe queueoutgoing tmpdir
+		       loop_command_max loop_command_sampling_delay loop_command_decrease_factor loop_prevention_regex
 		       purge_user_table_task  purge_orphan_bounces_task eval_bouncers_task process_bouncers_task
 		       minimum_bouncing_count minimum_bouncing_period bounce_delay 
 		       default_bounce_level1_rate default_bounce_level2_rate 
-		       remind_return_path request_priority rfc2369_header_fields sendmail sendmail_args sleep 
-		       sort sympa_priority syslog log_smtp umask welcome_return_path wwsympa_url
+		       remind_return_path request_priority return_path_suffix rfc2369_header_fields sendmail sendmail_args sleep 
+		       sort sympa_priority supported_lang syslog log_smtp umask verp_rate welcome_return_path wwsympa_url
                        openssl capath cafile  key_passwd ssl_cert_dir remove_headers
-		       antivirus_path antivirus_args antivirus_notify anonymous_header_fields
+		       antivirus_path antivirus_args antivirus_notify anonymous_header_fields sendmail_aliases
 		       dark_color light_color text_color bg_color error_color selected_color shaded_color
+		       color_0 color_1 color_2 color_3 color_4 color_5 color_6 color_7 color_8 color_9 color_10 color_11 color_12 color_13 color_14 color_15
+ 		       css_url css_path
 		       ldap_export_name ldap_export_host ldap_export_suffix ldap_export_password
-		       ldap_export_dnmanager ldap_export_connection_timeout urlize_min_size
+		       ldap_export_dnmanager ldap_export_connection_timeout update_db_field_types urlize_min_size
 		       list_check_smtp list_check_suffixes  spam_protection web_archive_spam_protection soap_url
+		       web_recode_to
 );
 
-my %old_options = ('trusted_ca_options' => 'capath,cafile');
+my %old_options = ('trusted_ca_options' => 'capath,cafile',
+		   'msgcat' => 'localedir',
+		   'queueexpire' => '');
 
 my %valid_options = ();
 map { $valid_options{$_}++; } @valid_options;
@@ -77,18 +84,21 @@ my %Default_Conf =
      'maxsmtp' => 20,
      'sendmail'=> '/usr/sbin/sendmail',
      'sendmail_args' => '-oi -odi -oem',
+     'sendmail_aliases' => '--SENDMAIL_ALIASES--',
      'openssl' => '',
      'host'    => undef,
      'domain'  => undef,
      'email'   => 'sympa',
      'pidfile' => '--PIDDIR--/sympa.pid',
-     'msgcat'  => '--NLSDIR--',
+     'pidfile_distribute' => '--PIDDIR--/sympa-distribute.pid',
+     'localedir'  => '--LOCALEDIR--',
      'sort'    => 'fr,ca,be,ch,uk,edu,*,com',
      'spool'   => '--SPOOLDIR--',
      'queue'   => undef,
+     'queuedistribute' => undef,
      'queuedigest'=> undef,
      'queuemod'   => undef,
-     'queueexpire'=> undef,
+     'queuetopic' => undef,
      'queueauth'  => undef,
      'queueoutgoing'  => undef,
      'queuebounce'  => undef,    
@@ -98,6 +108,8 @@ my %Default_Conf =
      'sleep'      => 5,
      'clean_delay_queue'    => 1,
      'clean_delay_queuemod' => 10,
+     'clean_delay_queuetopic' => 7,
+     'clean_delay_queuesubscribe' => 10,
      'clean_delay_queueauth' => 3,
      'log_socket_type'      => 'unix',
      'log_smtp'      => '',
@@ -111,14 +123,17 @@ my %Default_Conf =
      'db_options' => '',
      'db_env' => '',
      'db_port' => '',
+     'db_timeout' => '',
      'db_additional_subscriber_fields' => '',
      'db_additional_user_fields' => '',
+     'distribution_mode' => 'single',
      'listmaster' => undef,
+     'listmaster_email' => 'listmaster',
      'default_list_priority' => 5,
      'sympa_priority' => 1,
      'request_priority' => 0,
      'owner_priority' => 9,
-     'lang' => 'us',
+     'lang' => 'en_US',
      'misaddressed_commands' => 'reject',
      'misaddressed_commands_regexp' => '(subscribe|unsubscribe|signoff|set\s+(\S+)\s+(mail|nomail|digest))',
      'max_size' => 5242880,
@@ -128,24 +143,42 @@ my %Default_Conf =
      'wwsympa_url' => undef,
      'bounce_warn_rate' => '30',
      'bounce_halt_rate' => '50',
+     'bounce_email_prefix' => 'bounce',
      'cookie' => undef,
      'cookie_cas_expire' => '6',
      'loop_command_max' => 200,
      'loop_command_sampling_delay' => 3600,
      'loop_command_decrease_factor' => 0.5,
+     'loop_prevention_regex' => 'mailer-daemon|sympa|listserv|majordomo|smartlist|mailman',
      'rfc2369_header_fields' => 'help,subscribe,unsubscribe,post,owner,archive',
      'remove_headers' => 'Return-Receipt-To,Precedence,X-Sequence,Disposition-Notification-To',
      'antivirus_path' => '',
      'antivirus_args' => '',
      'antivirus_notify' => 'sender',
      'anonymous_header_fields' => 'Sender,X-Sender,Received,Message-id,From,X-Envelope-To,Resent-From,Reply-To,Organization,Disposition-Notification-To,X-Envelope-From,X-X-Sender',
-     'dark_color' => '#330099',
-     'light_color' => '#ccccff',
+     'dark_color' => 'silver',
+     'light_color' => '#aaddff',
      'text_color' => '#000000',
-     'bg_color' => '#ffffff',
+     'bg_color' => '#ffffcc',
      'error_color' => '#ff6666',
-     'selected_color' => '#3366cc',
-     'shaded_color' => '#eeeeee',
+     'selected_color' => 'silver',
+     'shaded_color' => '#66cccc',
+     'color_0' => '#F0F0F0', # very light grey use in tables
+     'color_1' => '#999', # main menu button color                       
+     'color_2' => '#333', # font color                                   
+     'color_3' => '#929292', # top boxe and footer box bacground color   
+     'color_4' => 'silver', #  page backgound color                      
+     'color_5' => '#fff', # ??                                           
+     'color_6' => '#99ccff', # list menu current button                  
+     'color_7' => '#ff99cc', # eroorbackground color,          
+     'color_8' => '#3366CC', #                                           
+     'color_9' => '#DEE7F7',
+     'color_10' => '#777777', # inactive button
+     'color_11' => '#3366CC', #                                          
+     'color_12' => '#000',
+     'color_13' => '#ffffcc',                                        # input backgound  | transparent
+     'color_14' => '#000',
+     'color_15' => '#000',
      'chk_cert_expiration_task' => '',
      'crl_update_task' => '',
      'ldap_export_name' => '',
@@ -173,7 +206,17 @@ my %Default_Conf =
      'default_bounce_level1_rate' => 45,
      'default_bounce_level2_rate' => 75,
      'soap_url' => '',
-     'urlize_min_size' => 10240 ## 10Kb
+     'css_url' => '',
+     'css_path' => '',
+     'urlize_min_size' => 10240, ## 10Kb
+     'supported_lang' => 'de,cs,el,es,et_EE,en_US,fr,hu,it,ja_JP,nl,oc,pt_BR,sv,tr',
+     'web_recode_to' => '',
+     'default_remind_task' => '',
+     'update_db_field_types' => 'auto',
+     'logo_html_definition' => '',
+     'return_path_suffix' => '-owner',
+     'verp_rate' => '0%',
+     'cache_list_config' => 'none',
      );
    
 my $wwsconf;
@@ -209,7 +252,7 @@ sub load {
 	    }
 	    $o{$keyword} = [ $value, $line_num ];
 	}else {
-	    printf STDERR Msg(1, 3, "Malformed line %d in %s: %s"), $line_num, $config, $_;
+	    printf STDERR gettext("Error at line %d : %s"), $line_num, $config, $_;
 	    $config_err++;
 	}
     }
@@ -219,6 +262,9 @@ sub load {
     unless (defined $o{'wwsympa_url'}) {
 	$o{'wwsympa_url'}[0] = "http://$o{'host'}[0]/wws";
     }
+    unless (defined $o{'css_url'}) {
+	$o{'css_url'}[0] = "$o{'wwsympa_url'}[0]/css/";
+    }
 
     # 'host' and 'domain' are mandatory and synonime.$Conf{'host'} is
     # still wydly use even if the doc require domain.
@@ -235,11 +281,14 @@ sub load {
     unless (defined $o{'queuedigest'}) {
 	$o{'queuedigest'}[0] = "$spool/digest";
     }
+    unless (defined $o{'queuedistribute'}) {
+	$o{'queuedistribute'}[0] = "$spool/distribute";
+    }
     unless (defined $o{'queuemod'}) {
 	$o{'queuemod'}[0] = "$spool/moderation";
     }
-    unless (defined $o{'queueexpire'}) {
-	$o{'queueexpire'}[0] = "$spool/expire";
+    unless (defined $o{'queuetopic'}) {
+	$o{'queuetopic'}[0] = "$spool/topic";
     }
     unless (defined $o{'queueauth'}) {
 	$o{'queueauth'}[0] = "$spool/auth";
@@ -260,8 +309,13 @@ sub load {
     ## Check if we have unknown values.
     foreach $i (sort keys %o) {
 	next if ($valid_options{$i});
-	if ($old_options{$i}) {
-	    printf STDERR  "Line %d of sympa.conf, parameter %s is no more available, read documentation for new parameter(s) %s\n", $o{$i}[1], $i, $old_options{$i};
+	if (defined $old_options{$i}) {
+	    if ($old_options{$i}) {
+		printf STDERR  "Line %d of sympa.conf, parameter %s is no more available, read documentation for new parameter(s) %s\n", $o{$i}[1], $i, $old_options{$i};
+	    }else {
+		printf STDERR  "Line %d of sympa.conf, parameter %s is now obsolete\n", $o{$i}[1], $i;
+		next;
+	    }
 	}else {
 	    printf STDERR  "Line %d, unknown field: %s in sympa.conf\n", $o{$i}[1], $i;
 	}
@@ -278,8 +332,20 @@ sub load {
     }
 
     
-    my @array = &_load_auth();
-    $Conf{'auth_services'} = [@array];
+    my $robots_conf = &load_robots ;    
+    $Conf{'robots'} = $robots_conf ;
+    my $nrcpt_by_domain =  &load_nrcpt_by_domain ;
+    $Conf{'nrcpt_by_domain'} = $nrcpt_by_domain ;
+    
+    foreach my $robot (keys %{$Conf{'robots'}}) {
+	my $config;
+	unless ($config = &tools::get_filename('etc', 'auth.conf', $robot)) {
+	    &do_log('err',"_load_auth: Unable to find auth.conf");
+	    next;
+	}
+
+	$Conf{'auth_services'}{$robot} = &_load_auth($robot, $config);	
+    }
 
     if ($Conf{'ldap_export_name'}) {    
 	##Export
@@ -319,19 +385,10 @@ sub load {
     }else {
 	$Conf{'remove_headers'} = [split(/,/, $Conf{'remove_headers'})];
     }
-
-    if ($Conf{'db_env'}) {
-	my @raw = split /;/, $Conf{'db_env'};
-	my %cooked;
-	foreach my $env (@raw) {
-	    next unless ($env =~ /^\s*(\w+)\s*\=\s*(\S+)\s*$/);
-	    $cooked{$1} = $2;
-	}
-	$Conf{'db_env'} = \%cooked;
-    }
-
+    $Conf{'listmaster'} =~ s/\s//g ;
     @{$Conf{'listmasters'}} = split(/,/, $Conf{'listmaster'});
 
+    
     ## Set Regexp for accepted list suffixes
     if (defined ($Conf{'list_check_suffixes'})) {
 	$Conf{'list_check_regexp'} = $Conf{'list_check_suffixes'};
@@ -341,12 +398,47 @@ sub load {
     $Conf{'sympa'} = "$Conf{'email'}\@$Conf{'host'}";
     $Conf{'request'} = "$Conf{'email'}-request\@$Conf{'host'}";
     
-    my $robots_conf = &load_robots ;
-    
-    $Conf{'robots'} = $robots_conf ;
     return 1;
 }
 
+## load nrcpt file (limite receipient par domain
+sub load_nrcpt_by_domain {
+  my $config = $Conf{'etc'}.'/nrcpt_by_domain.conf';
+  my $line_num = 0;
+  my $config_err = 0;
+  my %nrcpt_by_domain ; 
+  my $valid_dom = 0;
+
+  
+
+
+  return undef unless (-f $config) ;
+  &do_log('notice',"load_nrcpt: loading $config");
+
+  ## Open the configuration file or return and read the lines.
+  unless (open(IN, $config)) {
+      printf STDERR  "load: Unable to open %s: %s\n", $config, $!;
+      return undef;
+  }
+  while (<IN>) {
+      $line_num++;
+      next if (/^\s*$/o || /^[\#\;]/o);
+      if (/^(\S+)\s+(\d+)$/io) {
+	  my($domain, $value) = ($1, $2);
+	  chomp $domain; chomp $value;
+	  $nrcpt_by_domain->{$domain} = $value;
+	  $valid_dom +=1;
+      }else {
+	  printf STDERR gettext("Error at line %d : %s"), $line_num, $config, $_;
+	  $config_err++;
+      }
+  } 
+  close(IN);
+  &do_log('debug',"load_nrcpt: loaded $valid_dom config lines from $config");
+  return ($nrcpt_by_domain);
+}
+
+
 ## load each virtual robots configuration files
 sub load_robots {
     
@@ -357,6 +449,7 @@ sub load_robots {
 				  host            => 1,
 				  wwsympa_url     => 1,
 				  'title'         => 1,
+				  logo_html_definition        => 1,
 				  lang            => 1,
 				  default_home    => 1,
 				  cookie_domain   => 1,
@@ -377,6 +470,15 @@ sub load_robots {
 				  bounce_level1_rate => 1,
 				  bounce_level2_rate => 1,
 				  soap_url => 1,
+				  css_url => 1,
+ 				  css_path => 1,
+ 				  color_0 => 1, color_1 => 1, color_2 => 1, color_3 => 1, color_4 => 1, color_5 => 1,color_6 => 1, 
+				  color_7 => 1, color_8 => 1, color_9 => 1,
+				  color_10 => 1, color_11 => 1, color_12 => 1,color_13 => 1, color_14 => 1, color_15 => 1,
+				  supported_lang => 1,
+				  default_shared_quota => 1,
+				  verp_rate => 1,
+				  loop_prevention_regex => 1,
 				  );
 
     ## Load wwsympa.conf
@@ -408,7 +510,11 @@ sub load_robots {
 		my($keyword, $value) = ($1, $2);
 		$value =~ s/\s*$//;
 		$keyword = lc($keyword);
-		$value = lc($value) unless ($keyword eq 'title');
+
+		## Not all parameters should be lowercased
+		## We should define which parameter needs to be lowercased
+		#$value = lc($value) unless ($keyword eq 'title' || $keyword eq 'logo_html_definition' || $keyword eq 'lang');
+
 		if ($valid_robot_key_words{$keyword}) {
 		    $robot_conf->{$robot}{$keyword} = $value;
 		    # printf STDERR "load robots config: $keyword = $value\n";
@@ -419,6 +525,7 @@ sub load_robots {
 	    }
 	}
 	# listmaster is a list of email separated by commas
+	$robot_conf->{$robot}{'listmaster'} =~ s/\s//g;
 	@{$robot_conf->{$robot}{'listmasters'}} = split(/,/, $robot_conf->{$robot}{'listmaster'})
 	    if $robot_conf->{$robot}{'listmaster'};
 
@@ -433,10 +540,12 @@ sub load_robots {
 	$robot_conf->{$robot}{'log_smtp'} ||= $Conf{'log_smtp'};
 	$robot_conf->{$robot}{'log_level'} ||= $Conf{'log_level'};
 	$robot_conf->{$robot}{'wwsympa_url'} ||= 'http://'.$robot_conf->{$robot}{'http_host'}.'/wws';
+	$robot_conf->{$robot}{'css_url'} ||= $robot_conf->{$robot}{'wwsympa_url'}.'/css';
 	$robot_conf->{$robot}{'sympa'} = $robot_conf->{$robot}{'email'}.'@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'request'} = $robot_conf->{$robot}{'email'}.'-request@'.$robot_conf->{$robot}{'host'};
 	$robot_conf->{$robot}{'cookie_domain'} ||= 'localhost';
 	#$robot_conf->{$robot}{'soap_url'} ||= $Conf{'soap_url'};
+	$robot_conf->{$robot}{'verp_rate'} ||= $Conf{'verp_rate'};
 
 	my ($host, $path);
 	if ($robot_conf->{$robot}{'http_host'} =~ /^([^\/]+)(\/.*)$/) {
@@ -467,6 +576,28 @@ sub load_robots {
     return ($robot_conf);
 }
 
+## Check required files and create them if required
+sub checkfiles_as_root {
+
+    ## Check aliases file
+    unless (-f $Conf{'sendmail_aliases'}) {
+	unless (open ALIASES, ">$Conf{'sendmail_aliases'}") {
+	    &do_log('err',"Failed to create aliases file %s", $Conf{'sendmail_aliases'});
+	    return undef;
+	}
+
+	print ALIASES "## This aliases file is dedicated to Sympa Mailing List Manager\n";
+	print ALIASES "## You should edit your sendmail.mc or sendmail.cf file to declare it\n";
+	close ALIASES;
+	&do_log('notice', "Created missing file %s", $Conf{'sendmail_aliases'});
+	`chown --USER-- $Conf{'sendmail_aliases'}`;
+	`chgrp --GROUP-- $Conf{'sendmail_aliases'}`;
+	chmod 0644, $Conf{'sendmail_aliases'}
+	
+    }
+
+}
+
 ## Check a few files
 sub checkfiles {
     my $config_err = 0;
@@ -480,7 +611,7 @@ sub checkfiles {
 	}
     }
     
-    foreach my $qdir ('spool','queue','queuedigest','queuemod','queueexpire','queueauth','queueoutgoing','queuebounce','queuesubscribe','queuetask','tmpdir')
+    foreach my $qdir ('spool','queue','queuedigest','queuemod','queuetopic','queueauth','queueoutgoing','queuebounce','queuesubscribe','queuetask','queuedistribute','tmpdir')
     {
 	unless (-d $Conf{$qdir}) {
 	    do_log('info', "creating spool $Conf{$qdir}");
@@ -499,12 +630,22 @@ sub checkfiles {
 		$config_err++;
 	    }
 	}
+    ## Also create distribute/bad/
+    unless (-d $Conf{'queuedistribute'}.'/bad') {
+	    do_log('info', "creating spool $Conf{'queuedistribute'}/bad");
+	    unless ( mkdir ($Conf{'queuedistribute'}.'/bad', 0775)) {
+		do_log('err', 'Unable to create spool %s', $Conf{'queuedistribute'}.'/bad');
+		$config_err++;
+	    }
+	}
 
     ## Check cafile and capath access
     if (defined $Conf{'cafile'} && $Conf{'cafile'}) {
 	unless (-f $Conf{'cafile'} && -r $Conf{'cafile'}) {
 	    &do_log('err', 'Cannot access cafile %s', $Conf{'cafile'});
-	    &List::send_notify_to_listmaster('cannot_access_cafile', $Conf{'domain'}, $Conf{'cafile'});
+	    unless (&List::send_notify_to_listmaster('cannot_access_cafile', $Conf{'domain'}, [$Conf{'cafile'}])) {
+		&do_log('err', 'Unable to send notify "cannot access cafile" to listmaster');	
+	    }
 	    $config_err++;
 	}
     }
@@ -512,11 +653,21 @@ sub checkfiles {
     if (defined $Conf{'capath'} && $Conf{'capath'}) {
 	unless (-d $Conf{'capath'} && -x $Conf{'capath'}) {
 	    &do_log('err', 'Cannot access capath %s', $Conf{'capath'});
-	    &List::send_notify_to_listmaster('cannot_access_capath', $Conf{'domain'}, $Conf{'capath'});
+	    unless (&List::send_notify_to_listmaster('cannot_access_capath', $Conf{'domain'}, [$Conf{'capath'}])) {
+		&do_log('err', 'Unable to send notify "cannot access capath" to listmaster');	
+	    }
 	    $config_err++;
 	}
     }
 
+    ## queuebounce and bounce_path pointing to the same directory
+    if ($Conf{'queuebounce'} eq $wwsconf->{'bounce_path'}) {
+	&do_log('err', 'Error in config : queuebounce and bounce_path parameters pointing to the same directory (%s)', $Conf{'queuebounce'});
+	unless (&List::send_notify_to_listmaster('queuebounce_and_bounce_path_are_the_same', $Conf{'domain'}, [$Conf{'queuebounce'}])) {
+	    &do_log('err', 'Unable to send notify "queuebounce_and_bounce_path_are_the_same" to listmaster');	
+	}
+	$config_err++;
+    }
 
     return undef if ($config_err);
     return 1;
@@ -527,11 +678,9 @@ sub checkfiles {
 
 sub _load_auth {
     
-    my $config;
-    unless ($config = &tools::get_filename('etc', 'auth.conf', $Conf{'domain'})) {
-	do_log('err',"_load_auth: Unable to find auth.conf");
-	return undef;
-    }
+    my $robot = shift;
+    my $config = shift;
+    &do_log('notice', 'Conf::_load_auth(%s)', $config);
 
     my $line_num = 0;
     my $config_err = 0;
@@ -541,7 +690,7 @@ sub _load_auth {
 
     my %valid_keywords = ('ldap' => {'regexp' => '.*',
 				     'negative_regexp' => '.*',
-				     'host' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
+				     'host' => '[\w\.\-]+(:\d+)?(\s*,\s*[\w\.\-]+(:\d+)?)*',
 				     'timeout' => '\d+',
 				     'suffix' => '.+',
 				     'bind_dn' => '.+',
@@ -568,7 +717,7 @@ sub _load_auth {
 				    'proxy_validate_path' => '.*',
 				    'auth_service_name' => '.*',
 				    'authentication_info_url' => 'http(s)?:/.*',
-				    'ldap_host' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
+				    'ldap_host' => '[\w\.\-]+(:\d+)?(\s*,\s*[\w\.\-]+(:\d+)?)*',
 				    'ldap_bind_dn' => '.+',
 				    'ldap_bind_password' => '.+',
 				    'ldap_timeout'=> '\d+',
@@ -583,8 +732,22 @@ sub _load_auth {
 			  'generic_sso' => {'service_name' => '.+',
 					    'service_id' => '\S+',
 					    'http_header_prefix' => '\w+',
-					    'email_http_header' => '\w+'
-					    }
+					    'email_http_header' => '\w+',
+					    'ldap_host' => '[\w\.\-]+(:\d+)?(\s*,\s*[\w\.\-]+(:\d+)?)*',
+					    'ldap_bind_dn' => '.+',
+					    'ldap_bind_password' => '.+',
+					    'ldap_timeout'=> '\d+',
+					    'ldap_suffix'=> '.+',
+					    'ldap_scope' => 'base|one|sub',
+					    'ldap_get_email_by_uid_filter' => '.+',
+					    'ldap_email_attribute' => '\w+',
+					    'ldap_use_ssl' => '1',
+					    'ldap_ssl_version' => 'sslv2/3|sslv2|sslv3|tlsv1',
+					    'ldap_ssl_ciphers' => '[\w:]+',
+					    'force_email_verify' => '1',
+					    'internal_email_by_netid' => '1',
+					    'netid_http_header' => '\w+',
+					}
 			  );
     
 
@@ -595,10 +758,10 @@ sub _load_auth {
 	return undef;
     }
     
-    $Conf{'cas_number'} = 0;
-    $Conf{'generic_sso_number'} = 0;
-    $Conf{'ldap_number'} = 0;
-    $Conf{'use_passwd'} = 0;
+    $Conf{'cas_number'}{$robot} = 0;
+    $Conf{'generic_sso_number'}{$robot} = 0;
+    $Conf{'ldap_number'}{$robot} = 0;
+    $Conf{'use_passwd'}{$robot} = 0;
     
     ## Parsing  auth.conf
     while (<IN>) {
@@ -611,13 +774,18 @@ sub _load_auth {
 	}elsif (/^\s*(\S+)\s+(.*\S)\s*$/o){
 	    my ($keyword,$value) = ($1,$2);
 	    unless (defined $valid_keywords{$current_paragraph->{'auth_type'}}{$keyword}) {
-		do_log('notice',"_load_auth: unknown keyword '%s' in %s line %d", $keyword, $config, $line_num);
+		do_log('err',"_load_auth: unknown keyword '%s' in %s line %d", $keyword, $config, $line_num);
 		next;
 	    }
 	    unless ($value =~ /^$valid_keywords{$current_paragraph->{'auth_type'}}{$keyword}$/) {
-		do_log('notice',"_load_auth: unknown format '%s' for keyword '%s' in %s line %d", $value, $keyword, $config,$line_num);
+		do_log('err',"_load_auth: unknown format '%s' for keyword '%s' in %s line %d", $value, $keyword, $config,$line_num);
 		next;
 	    }
+
+	    ## Allow white spaces between hosts
+	    if ($keyword =~ /host$/) {
+		$value =~ s/\s//g;
+	    }
 	    
 	    $current_paragraph->{$keyword} = $value;
 	}
@@ -654,16 +822,16 @@ sub _load_auth {
 			next;
 		    }
 
-		    $Conf{'cas_number'}  ++ ;
-		    $Conf{'cas_id'}{$current_paragraph->{'auth_service_name'}} =  $#paragraphs+1 ; 
+		    $Conf{'cas_number'}{$robot}  ++ ;
+		    $Conf{'cas_id'}{$robot}{$current_paragraph->{'auth_service_name'}} =  $#paragraphs+1 ; 
 		}elsif($current_paragraph->{'auth_type'} eq 'generic_sso') {
-		    $Conf{'generic_sso_number'}  ++ ;
-		    $Conf{'generic_sso_id'}{$current_paragraph->{'service_id'}} =  $#paragraphs+1 ; 
+		    $Conf{'generic_sso_number'}{$robot}  ++ ;
+		    $Conf{'generic_sso_id'}{$robot}{$current_paragraph->{'service_id'}} =  $#paragraphs+1 ; 
 		}elsif($current_paragraph->{'auth_type'} eq 'ldap') {
-		    $Conf{'ldap'}  ++ ;
-		    $Conf{'use_passwd'} = 1;
+		    $Conf{'ldap'}{$robot}  ++ ;
+		    $Conf{'use_passwd'}{$robot} = 1;
 		}elsif($current_paragraph->{'auth_type'} eq 'user_table') {
-		    $Conf{'use_passwd'} = 1;
+		    $Conf{'use_passwd'}{$robot} = 1;
 		}
 		# setting default
 		$current_paragraph->{'regexp'} = '.*' unless (defined($current_paragraph->{'regexp'})) ;
@@ -677,7 +845,7 @@ sub _load_auth {
     }
     close(IN); 
 
-    return @paragraphs;
+    return \@paragraphs;
     
 }
 
