@@ -1,5 +1,5 @@
 # Ldap.pm - This module includes most LDAP-related functions
-# RCS Identication ; $Revision: 1.9 $ ; $Date: 2004/03/03 15:06:02 $ 
+# RCS Identication ; $Revision: 1.14 $ ; $Date: 2006/01/05 14:23:29 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -28,17 +28,23 @@ require Exporter;
 @ISA = qw(Exporter);
 @EXPORT = qw(%Ldap);
 
-my @valid_options = qw(host port suffix filter scope);
+my @valid_options = qw(host port suffix filter scope bind_dn bind_password);
+my  @required_options = qw(host port suffix filter scope);
 
 my %valid_options = ();
 map { $valid_options{$_}++; } @valid_options;
 
+my %required_options = ();
+map { $required_options{$_}++; } @required_options;
+
 my %Default_Conf =
     ( 	'host'=> undef,
     	'port' => undef,
     	'suffix' => undef,
     	'filter' => undef,
-    	'scope' => undef
+    	'scope' => undef,
+	'bind_dn' => undef,
+	'bind_password' => undef
    );
 
 %Ldap = ();
@@ -77,19 +83,20 @@ sub load {
 
     ## Check if we have unknown values.
     foreach $i (sort keys %o) {
-	next if ($valid_options{$i});
-	&Log::do_log('err',"Line %d, unknown field: %s \n", $o{$i}[1], $i);
-	$config_err++;
+	$Ldap{$i} = $o{$i}[0] || $Default_Conf{$i};
+	
+	unless ($valid_options{$i}) {
+	    &Log::do_log('err',"Line %d, unknown field: %s \n", $o{$i}[1], $i);
+	    $config_err++;
+	}
     }
     ## Do we have all required values ?
-    foreach $i (keys %valid_options) {
+    foreach $i (keys %required_options) {
 	unless (defined $o{$i} or defined $Default_Conf{$i}) {
 	    &Log::do_log('err',"Required field not found : %s\n", $i);
 	    $config_err++;
 	    next;
 	}
-	$Ldap{$i} = $o{$i}[0] || $Default_Conf{$i};
-	
     }
  return %Ldap;
 }
@@ -103,7 +110,8 @@ sub export_list{
 
     ##To record owner's and editor's email and gecos
     ## !! STRUCTURE LDAP A REVOIR
-    foreach my $element (@{$list->{'admin'}{'owner'}}){
+    my $owners = $list->get_owners();
+    foreach my $element (@{$owners}) {
 	next unless (defined $element->{'email'});
 	if (ref($element->{'email'})) {
 	    push(@owner_emails, @{$element->{'email'}});
@@ -146,7 +154,7 @@ sub export_list{
     return undef
 	unless &delete_list($directory, $list, $ldap);
 	
-    my $list_email = "$list->{'name'}".'@'."$list->{'admin'}{'host'}"; 
+    my $list_email = $list->get_list_address();
     my $dn = "cn=$list_email,$Conf{'ldap_export'}{$directory}{'suffix'}";
 
     my $total =  $list->get_total() || 0;
@@ -227,7 +235,7 @@ sub delete_list{
     }
     
     ##To create the dn and delete this entry
-    my $list_email = "$list->{'name'}".'@'."$list->{'admin'}{'host'}"; 
+    my $list_email = $list->get_list_address();
     my $dn = "cn=$list_email,$Conf{'ldap_export'}{$directory}{'suffix'}";
     my $filter = "(listEmailAddress = $list_email)";
     
@@ -273,7 +281,13 @@ sub get_exported_lists{
 	return undef;
     }
    
-    $ldap->bind();
+    my $cnx = $ldap->bind();
+	
+    unless(defined($cnx) && ($cnx->code == 0)){
+	&Log::do_log('err', 'Ldap::exported_lists: Bind failed on  %s',$Conf{'ldap_export'}{$directory}{'host'});
+	$ldap->unbind;
+	return undef;
+    }
 	
     my $search_filter = "(|(listEmailAddress=*$filter*)(listSubject=*$filter*))";
     my $result_search = $ldap->search (
@@ -326,7 +340,14 @@ sub get_dn_anonymous{
     }
 
     ##Bind
-    $ldap->bind();
+    my $cnx = $ldap->bind();
+
+    unless(defined($cnx) && ($cnx->code == 0)){
+	&Log::do_log('err', 'Ldap::exported_lists: Bind failed on  %s',$datas->{'host'});
+	$ldap->unbind;
+	return undef;
+    }
+    
 
     ##Search
     my $result_search = $ldap->search (
