@@ -1,7 +1,7 @@
 #! --PERL--
 
 # Log.pm - This module includes all Logging-related functions
-# RCS Identication ; $Revision: 1.15 $ ; $Date: 2004/02/20 09:53:02 $ 
+# RCS Identication ; $Revision: 1.19 $ ; $Date: 2006/03/24 17:00:31 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -44,7 +44,10 @@ sub fatal_err {
     my $full_msg = sprintf $m,@_;
 
     ## Notify listmaster
-    &List::send_notify_to_listmaster('sympa_died', $Conf{'domain'}, $m);
+    unless (&List::send_notify_to_listmaster('sympa_died', $Conf::Conf{'host'}, [$full_msg])) {
+	&do_log('err',"Unable to send notify 'sympa died' to listmaster");
+    }
+
 
     printf STDERR "$m\n", @_;
     exit(1);   
@@ -74,8 +77,8 @@ sub do_log {
 	    syslog($fac, $m, @_);
     }
     if ($main::options{'foreground'}) {
-	if (!$main::options{'batch'} 
-	    || ($main::options{'batch'} && $fac eq 'err')) {
+	if ($main::options{'log_to_stderr'} || 
+	    ($main::options{'batch'} && $fac eq 'err')) {
 	    $m =~ s/%m/$errno/g;
 	    printf STDERR "$m\n", @_;
 	}
@@ -100,6 +103,8 @@ sub do_connect {
     if ($log_socket_type =~ /^(unix|inet)$/i) {
       Sys::Syslog::setlogsock(lc($log_socket_type));
     }
+    # close log may be usefull : if parent processus did open log child process inherit the openlog with parameters from parent process 
+    closelog ; 
     openlog("$log_service\[$$\]", 'ndelay', $log_facility);
 }
 
