@@ -1,5 +1,5 @@
-# Message.pm - This module includes all list processing functions
-# <!-- RCS Identication ; $Revision: 1.10 $ ; $Date: 2004/03/05 09:25:04 $ -->
+# Message.pm - This module includes Message processing functions
+#<!-- RCS Identication ; $Revision: 1.18 $ ; $Date: 2006/01/20 16:11:15 $ --> 
 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
@@ -25,7 +25,7 @@ package Message;
 use strict;
 require Exporter;
 require 'tools.pl';
-require 'parser.pl';
+require 'tt2.pl';
 my @ISA = qw(Exporter);
 my @EXPORT = qw();
 
@@ -43,9 +43,9 @@ use Log;
 
 ## Creates a new object
 sub new {
-    my($pkg, $file) = @_;
+    my($pkg, $file, $noxsympato) = @_;
     my $message;
-    &do_log('debug2', 'Message::new(%s)', $file);
+    &do_log('debug2', 'Message::new(%s,%s)',$file,$noxsympato);
     
     if (ref($file) =~ /MIME::Entity/i) {
 	$message->{'msg'} = $file;
@@ -73,6 +73,7 @@ sub new {
 	return undef;
     }
     $message->{'msg'} = $msg;
+    $message->{'msg_as_string'} = $msg->as_string;
     
     ## Message size
     $message->{'size'} = -s $file;    
@@ -92,41 +93,56 @@ sub new {
     }
     $message->{'sender'} = lc($sender_hdr[0]->address);
 
+    unless (&tools::valid_email($message->{'sender'})) {
+	do_log('err', "Invalid From: field '%s'", $message->{'sender'});
+	return undef;
+    }
+
+    ## Store decoded subject
+    my @decoded_subject =  &MIME::Words::decode_mimewords($hdr->get('Subject'));
+    foreach my $token (@decoded_subject) {
+	$message->{'decoded_subject'} .= $token->[0]; 
+	$message->{'subject_charset'} ||= $token->[1]; 
+    }
+    chomp $message->{'decoded_subject'};
+
     ## Extract recepient address (X-Sympa-To)
     $message->{'rcpt'} = $hdr->get('X-Sympa-To');
     chomp $message->{'rcpt'};
-    unless ($message->{'rcpt'}) {
-	do_log('err', 'no X-Sympa-To found, ignoring message file %s', $file);
-	return undef;
-    }
-    
-    ## Strip of the initial X-Sympa-To field
-    # Used by checksum later
-    #$hdr->delete('X-Sympa-To');
-
-    ## Do not check listname if processing a web message
-    unless ($hdr->get('X-Sympa-From')) {
-	## get listname & robot
-	my ($listname, $robot) = split(/\@/,$message->{'rcpt'});
+    unless (defined $noxsympato) { # message.pm can be used for mesage not in the spool but in archive
+	unless ($message->{'rcpt'}) {
+	    do_log('err', 'no X-Sympa-To found, ignoring message file %s', $file);
+	    return undef;
+	}
 	
-	$robot = lc($robot);
-	$listname = lc($listname);
-	$robot ||= $Conf{'host'};
+	## Strip of the initial X-Sympa-To field
+	# Used by checksum later
+	#$hdr->delete('X-Sympa-To');
 	
-	my $conf_email = &Conf::get_robot_conf($robot, 'email');
-	my $conf_host = &Conf::get_robot_conf($robot, 'host');
-	unless ($listname =~ /^(sympa|listmaster|$conf_email)(\@$conf_host)?$/i) {
-	    my $list_check_regexp = &Conf::get_robot_conf($robot,'list_check_regexp');
+	## Do not check listname if processing a web message
+	unless ($hdr->get('X-Sympa-From')) {
+	    ## get listname & robot
+	    my ($listname, $robot) = split(/\@/,$message->{'rcpt'});
+	    
+	    $robot = lc($robot);
+	    $listname = lc($listname);
+	    $robot ||= $Conf{'host'};
+	    
+	    my $conf_email = &Conf::get_robot_conf($robot, 'email');
+	    my $conf_host = &Conf::get_robot_conf($robot, 'host');
+	    unless ($listname =~ /^(sympa|$Conf{'listmaster_email'}|$conf_email)(\@$conf_host)?$/i) {
+		my $list_check_regexp = &Conf::get_robot_conf($robot,'list_check_regexp');
 	        if ($listname =~ /^(\S+)-($list_check_regexp)$/) {
 		    $listname = $1;
 		}
-	    
-	    $message->{'list'} = new List ($listname, $robot);
-	    unless ($message->{'rcpt'}) {
-		do_log('err', 'Could not create List object for list %s in robot %s', $listname, $robot);
-		return undef;
+		
+		$message->{'list'} = new List ($listname, $robot);
+		unless ($message->{'rcpt'}) {
+		    do_log('err', 'Could not create List object for list %s in robot %s', $listname, $robot);
+		    return undef;
+		}
+		
 	    }
-
 	}
     }
 
@@ -167,6 +183,12 @@ sub new {
 	
     }
 
+    ## TOPICS
+    my $topics;
+    if ($topics = $hdr->get('X-Sympa-Topic')){
+	$message->{'topic'} = $topics;
+    }
+
     ## Bless Message object
     bless $message, $pkg;
 
@@ -195,5 +217,30 @@ sub dump {
     return 1;
 }
 
+## Add topic and put header X-Sympa-Topic
+sub add_topic {
+    my ($self,$topic) = @_;
+
+    $self->{'topic'} = $topic;
+    my $hdr = $self->{'msg'}->head;
+    $hdr->add('X-Sympa-Topic', $topic);
+
+    return 1;
+}
+
+
+## Get topic
+sub get_topic {
+    my ($self) = @_;
+
+    if (defined $self->{'topic'}) {
+	return $self->{'topic'};
+
+    } else {
+	return '';
+    }
+}
+
+
 ## Packages must return true.
 1;
