@@ -1,8 +1,12 @@
 #!--PERL--
 # alias_manager.pl -  this script is intended to create automatically list aliases
 # when using sympa. Aliases can be added or removed in file --SENDMAIL_ALIASES--
-# RCS Identication ; $Revision: 1.21 $ ; $Date: 2003/07/10 13:00:17 $ 
-#
+# RCS Identication ; $Revision: 1.34 $ ; $Date: 2006/01/04 13:16:30 $ 
+
+# L. Marcotte has written a version of alias_manager.pl that is LDAP enabled
+# check the contrib. page for more information :
+# http://sympa.org/contrib.html
+
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
 # Copyright (c) 1997,1998, 1999 Institut Pasteur & Christophe Wolfhugel
@@ -29,11 +33,11 @@ use lib '--LIBDIR--';
 use Conf;
 use POSIX;
 require "tools.pl";
-require "parser.pl";
+require "tt2.pl";
 
 
 unless (Conf::load('--CONFIG--')) {
-   print Msg(1, 1, "Configuration file --CONFIG-- has errors.\n");
+   print STDERR gettext("The configuration file --CONFIG-- contains errors.\n");
    exit(1);
 }
 my $tmp_alias_file = $Conf{'tmpdir'}.'/sympa_aliases.'.time;
@@ -50,13 +54,14 @@ my ($operation, $listname, $domain, $file) = @ARGV;
 
 
 if (($operation !~ /^(add)|(del)$/) || ($#ARGV < 2)) {
-    printf "Usage: $0 <add|del> <listname> <domain> [<file>]\n";
+    printf STDERR "Usage: $0 <add|del> <listname> <robot> [<file>]\n";
     exit(2);
 }
 
 $default_domain = $Conf{'domain'};
 
-my $alias_file = '--SENDMAIL_ALIASES--';
+my $alias_file;
+$alias_file = $Conf{'sendmail_aliases'} || '--SENDMAIL_ALIASES--';
 $alias_file = $file if ($file);
 
 unless (-w "$alias_file") {
@@ -72,11 +77,23 @@ $data{'list'}{'domain'} = $data{'robot'} = $domain;
 $data{'list'}{'name'} = $listname;
 $data{'default_domain'} = $default_domain;
 $data{'is_default_domain'} = 1 if ($domain eq $default_domain);
-my $template_file = &tools::get_filename('etc', 'list_aliases.tpl', $domain);
-my @aliases ;
-&parser::parse_tpl (\%data,$template_file,\@aliases);
+$data{'return_path_suffix'} = &Conf::get_robot_conf($domain, 'return_path_suffix');
 
 
+my @aliases ;
+
+my $tt2_include_path = &tools::make_tt2_include_path($domain,'',,);
+
+my $aliases_dump;
+&tt2::parse_tt2 (\%data, 'list_aliases.tt2',\$aliases_dump, $tt2_include_path);
+
+@aliases = split /\n/, $aliases_dump;
+
+unless (@aliases) {
+    	print STDERR "No aliases defined\n";
+	exit(15);
+}
+
 if ($operation eq 'add') {
     ## Create a lock
     unless (open(LF, ">>$lock_file")) { 
@@ -97,12 +114,12 @@ if ($operation eq 'add') {
     }
 
     foreach (@aliases) {
-	print ALIAS "$_";
+	print ALIAS "$_\n";
     }
     close ALIAS;
 
     ## Newaliases
-    if ($alias_file eq '--SENDMAIL_ALIASES--') {
+    unless ($file) {
 	unless (system($alias_wrapper) == 0) {
 	    print STDERR "Failed to execute newaliases: $!\n";
 	    exit(6)
@@ -173,7 +190,7 @@ if ($operation eq 'add') {
     unlink $tmp_alias_file;
 
     ## Newaliases
-    if ($alias_file eq '--SENDMAIL_ALIASES--') {
+    unless ($file) {
 	unless (system($alias_wrapper) == 0) {
 	    print STDERR "Failed to execute newaliases: $!\n";
 	exit (6);
