@@ -38,7 +38,7 @@ foreach $msg ( sort grep(!/^\./, readdir SPOOL )) {
     next if ($msg =~ /^\./);
     
     $msg =~ /^(.*)\_([^\_]+)$/;
-    my ($listname, $modkey) = ($1, $2);
+    my ($listaddress, $modkey) = ($1, $2);
     
     
     if (-d "$Conf{'queuemod'}/.$msg") {
@@ -47,7 +47,8 @@ foreach $msg ( sort grep(!/^\./, readdir SPOOL )) {
     
     print "Creating HTML version for $Conf{'queuemod'}/$msg\n";
     
-    my $self = new List ($listname);
+    my ($listname, $listrobot) = split /\@/, $listaddress;
+    my $self = new List ($listname, $listrobot);
     
     my( @rcpt);
     my $admin = $self->{'admin'};
@@ -56,7 +57,7 @@ foreach $msg ( sort grep(!/^\./, readdir SPOOL )) {
     my $robot = $self->{'domain'};
     my $modqueue = $Conf{'queuemod'};
     unless ($name && $admin) {
-	print STDERR "Unkown list $listname, skipping\n";
+	print STDERR "Unkown list $listaddress, skipping\n";
 	next;
     }
     
@@ -65,7 +66,7 @@ foreach $msg ( sort grep(!/^\./, readdir SPOOL )) {
 	unless (mkdir ($tmp_dir, 0777)) {
 	    die "May not create $tmp_dir";
 	}
-	my $mhonarc_ressources = &tools::get_filename('etc', 'mhonarc-ressources', $robot, $self);
+	my $mhonarc_ressources = &tools::get_filename('etc', 'mhonarc-ressources.tt2', $robot, $self);
 	unless ($mhonarc_ressources) {
 	    die "Cannot find any MhOnArc ressource file";
 	}
