@@ -3,7 +3,7 @@
 # p12topem.pl - This script installs a List X509 cert and
 # the associated private key in list directory
 # Input is PKCS#12 file
-# RCS Identication ; $Revision: 1.5.4.1 $ ; $Date: 2004/06/30 15:42:02 $ 
+# RCS Identication ; $Revision: 1.7 $ ; $Date: 2005/08/12 15:46:18 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -73,7 +73,7 @@ if (($main::options{'help'} ne '') ||
 }else{
 
     if ($listname) {
-	my $self = new List($listname);
+	my $self = new List($listname, $robot);
 	$cert = $self->{'dir'}.'/cert.pem';
 	$privatekey = $self->{'dir'}.'/private_key';
 	unless (-d $self->{'dir'}) {
