@@ -1,5 +1,5 @@
 /* queue.c - This program does the messages spooling
-   RCS Identication ; $Revision: 1.4.2.1 $ ; $Date: 2005/01/27 08:54:02 $ 
+   RCS Identication ; $Revision: 1.7 $ ; $Date: 2006/03/21 09:59:04 $ 
 
    Sympa - SYsteme de Multi-Postage Automatique
    Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -26,7 +26,7 @@
 #include <string.h>
 #include <stdlib.h>
 
-static char rcsid[] = "(@)$Id: queue.c,v 1.4.2.1 2005/01/27 08:54:02 sympa-authors Exp $";
+static char rcsid[] = "(@)$Id: queue.c,v 1.7 2006/03/21 09:59:04 sympa-authors Exp $";
 
 static char     qfile[128];
 static char     buf[16384];
@@ -41,11 +41,11 @@ static int      i, fd;
 # define CONFIG		"/etc/sympa.conf"
 #endif
 
-unsigned char *
+char *
 readconf(char *file)
 {
    FILE			*f;
-   unsigned char	buf[16384], *p, *r, *s;
+   char	buf[16384], *p, *r, *s;
 
    r = NULL;
    if ((f = fopen(file, "r")) != NULL) {
@@ -53,7 +53,7 @@ readconf(char *file)
 	/* Search for the configword "queue" and a whitespace after it */
 	if (strncmp(buf, "queue", 5) == 0 && isspace(buf[5])) {
             /* Strip the ending \n */
-            if ((p = (unsigned char *)strrchr((char *)buf, '\n')) != NULL)
+            if ((p = strrchr((char *)buf, '\n')) != NULL)
                *p = '\0';
             p = buf + 5;
             while (*p && isspace(*p)) p++;
@@ -84,8 +84,8 @@ readconf(char *file)
 int
 main(int argn, char **argv)
 {
-   unsigned char	*queuedir;
-   unsigned char        *listname;
+   char	*queuedir;
+   char        *listname;
    unsigned int		priority;
    int			firstfrom = 0;
 
