@@ -103,8 +103,8 @@ my @params = ({'title' => 'Directories and file location'},
 	       'file' => 'wwsympa.conf',
                'advice' =>'Better if not in a critical partition'},
 	      
-	      {'name' => 'msgcat',
-	       'default' => '--NLSDIR--',
+	      {'name' => 'localedir',
+	       'default' => '--LOCALEDIR--',
 	       'query' => 'Directory containing available NLS catalogues (Message internationalization)',
 	       'file' => 'sympa.conf',
 	       'advice' =>''},
@@ -161,9 +161,9 @@ my @params = ({'title' => 'Directories and file location'},
 	      
 	      {'name' => 'listmaster',
 	       'default' => 'your_email_address@--HOST--',
-	       'query' => 'Listmasters email list colon separated',
+	       'query' => 'Listmasters email list comma separated',
 	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
+	       'advice' =>'Sympa will associate listmaster privileges to these email addresses (mail and web interfaces). Some error reports may also be sent to these addresses.'},
 	      
 	      {'name' => 'email',
 	       'default' => 'sympa',
@@ -172,8 +172,8 @@ my @params = ({'title' => 'Directories and file location'},
 	       'advice' =>"Effective address will be \[EMAIL\]@\[HOST\]"},
 
 	      {'name' => 'lang',
-	       'default' => 'us',
-	       'query' => 'Default lang (fr | us | es | de | it | cn | tw | fi | pl | cz | hu | ro | et)',
+	       'default' => 'en_US',
+	       'query' => 'Default lang (cs | de | el | en_US | fr | hu | it | ja_JP | nl | oc | pt_BR | tr)',
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>''},
 
@@ -183,7 +183,15 @@ my @params = ({'title' => 'Directories and file location'},
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>'This parameter is a scenario, check sympa documentation about scenarios if you want to define one'},
 
-	      
+	      {'title' => 'Tuning'},
+	      	      
+
+	      {'name' => 'cache_list_config',
+	       'default' => 'none',
+	       'query' => 'Use of binary version of the list config structure on disk: none | binary_file',
+	       'file' => 'sympa.conf','edit' => '1',
+	       'advice' =>'Set this parameter to "binary_file" if you manage a big amount of lists (1000+) ; it should make the web interface startup faster'},
+
 	      {'name' => 'sympa_priority',
 	       'query' => 'Sympa commands priority',
 	       'file' => 'sympa.conf',
@@ -275,7 +283,7 @@ my @params = ({'title' => 'Directories and file location'},
 
 	      {'name' => 'nrcpt',
 	       'default' => '25',
-	       'query' => 'Maximum number of recipients per call to Sendmail',
+	       'query' => 'Maximum number of recipients per call to Sendmail. The nrcpt_by_domain.conf file allows a different tuning per destination domain.',
 	       'file' => 'sympa.conf',
 	       'advice' =>''},
 
@@ -344,16 +352,16 @@ my @params = ({'title' => 'Directories and file location'},
 	      {'title' => 'Database'},
 	      
 	      {'name' => 'db_type',
-	       'sample' => 'mysql',
-	       'query' => 'Database type (mysql | Pg | Oracle | Sybase)',
+	       'default' => 'mysql',
+	       'query' => 'Database type (mysql | Pg | Oracle | Sybase | SQLite)',
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>'be carefull to the case'},
 
 	      {'name' => 'db_name',
-	       'sample' => 'sympa',
+	       'default' => 'sympa',
 	       'query' => 'Name of the database',
 	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
+	       'advice' =>'with SQLite, the name of the DB corresponds to the DB file'},
 
 	      {'name' => 'db_host',
 	       'sample' => 'localhost',
@@ -404,7 +412,7 @@ my @params = ({'title' => 'Directories and file location'},
 	       'advice' =>'This module provide much faster web interface'},
 
 	      {'name' => 'wwsympa_url',
-	       'default' => 'http://--HOST--/wws',
+	       'default' => 'http://--HOST--/sympa',
 	       'query' => "Sympa\'s main page URL",
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>''},
@@ -416,7 +424,7 @@ my @params = ({'title' => 'Directories and file location'},
 	       'advice' =>''},
 
 	      {'name' => 'icons_url',
-	       'default' => '/icons/sympa',
+	       'default' => '/icons',
 	       'query' => 'Icons directory (web) location for Sympa',
 	       'file' => 'wwsympa.conf'},
 
@@ -431,36 +439,6 @@ my @params = ({'title' => 'Directories and file location'},
 	       'file' => 'sympa.conf','edit' => '1',
 	       'advice' =>''},
 
-	      {'name' => 'dark_color',
-	       'default' => '#006666',
-	       'query' => 'web interface color : dark',
-	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
-	      
-	      {'name' => 'selected_color',
-	       'default' => '#996666',
-	       'query' => 'web interface color : selected_color',
-	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
-	      
-	      {'name' => 'light_color',
-	       'default' => '#cccc66',
-	       'query' => 'web interface color : light',
-	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
-	      
-	      {'name' => 'shaded_color',
-	       'default' => '#66cccc',
-	       'query' => 'web_interface color : shaded',
-	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
-	      
-	      {'name' => 'bg_color',
-	       'default' => '#ffffcc',
-	       'query' => 'web_interface color : background',
-	       'file' => 'sympa.conf','edit' => '1',
-	       'advice' =>''},
-	      
 	      );
 
 
