@@ -1,7 +1,7 @@
 #! --PERL--
 
 # task_manager.pl - This script runs as a daemon and processes periodical Sympa tasks
-# RCS Identication ; $Revision: 1.57.2.2 $ ; $Date: 2004/09/16 08:57:25 $ 
+# RCS Identication ; $Revision: 1.89.2.1 $ ; $Date: 2006/04/19 13:21:17 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -28,15 +28,17 @@
 use lib '--LIBDIR--';
 use strict vars;
 
+use Task;
 use List;
 use Conf;
 use Log;
 use Getopt::Long;
 use Time::Local;
 use Digest::MD5;
-use smtp;
+use mail;
 use wwslib;
  
+require 'tt2.pl';
 require 'parser.pl';
 require 'tools.pl';
 
@@ -79,7 +81,14 @@ unless (Conf::load($sympa_conf_file)) {
 }
 
 ## Check databse connectivity
-$List::use_db = &List::probe_db();
+unless ($List::use_db = &List::check_db_connect()) {
+    &fatal_err('Database %s defined in sympa.conf has not the right structure or is unreachable. If you don\'t use any database, comment db_xxx parameters in sympa.conf', $Conf{'db_name'});
+}
+
+## Check that the data structure is uptodate
+unless (&List::data_structure_uptodate()) {
+    &fatal_err("error : data structure was not updated ; you should run sympa.pl to run the upgrade process.");
+}
 
 ## Check for several files.
 unless (&Conf::checkfiles()) {
@@ -131,11 +140,9 @@ $< = $> = (getpwnam('--USER--'))[2];
 ## Sets the UMASK
 umask(oct($Conf{'umask'}));
 
-&Language::LoadLang($Conf{'msgcat'});
-
 ## Change to list root
 unless (chdir($Conf{'home'})) {
-    &message('chdir_error');
+    &report::reject_report_web('intern','chdir_error',{},'','','', $Conf{'host'});
     &do_log('err',"error : unable to change to directory $Conf{'home'}");
     exit (-1);
 }
@@ -234,19 +241,17 @@ while (!$end) {
     my $current_date = time; # current epoch date
     my $rep = &tools::adate ($current_date);
 
-    ## Create required tasks
-    unless (opendir(DIR, $spool_task)) {
-	&do_log ('err', "error : can't open dir %s: %m", $spool_task);
+
+    ## List all tasks
+    unless (&Task::list_tasks($spool_task)) {
+	&List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list task files in $spool_task"})
+	&do_log ('err', "Failed to list task files in %s", $spool_task);
+	exit -1;
     }
-    undef @tasks;
-    @tasks = sort epoch_sort (grep !/^\.\.?$/, readdir DIR); # @tasks updating
-    closedir DIR;
 
     my %used_models; # models for which a task exists
-    foreach (@tasks) {
-	if (my $task = &match_task($_)) {
-	    $used_models{$task->{'model'}} = 1;
-	}
+    foreach my $model (&Task::get_used_models) {
+	$used_models{$model} = 1;
     }
 
     ### creation of required tasks 
@@ -259,7 +264,7 @@ while (!$end) {
 	    if ($Conf{$key}) { 
 		my %data = %default_data; # hash of datas necessary to the creation of tasks
 		#printf "xxxxxxxxxxxxx appel 1\n";
-		create ($current_date, '', $global_models{$key}, $Conf{$key}, '_global', \%data);
+		create ($current_date, '', $global_models{$key}, $Conf{$key}, \%data);
 		$used_models{$1} = 1;
 	    }
 	}
@@ -267,39 +272,38 @@ while (!$end) {
 
     
     ## list tasks
-    foreach ( &List::get_lists() ) {
+    foreach my $robot (keys %{$Conf{'robots'}}) {
+	my $all_lists = &List::get_lists($robot);
+	foreach my $list ( @$all_lists ) {
+	    
+	    my %data = %default_data;
 	
-	my %data = %default_data;
-	my $list = new List ($_);
+	    $data{'list'} = {'name' => $list->{'name'},
+			     'robot' => $list->{'domain'}};
 	
-	$data{'list'}{'name'} = $list->{'name'};
-	
-	my %used_list_models; # stores which models already have a task 
-	foreach (@list_models) { $used_list_models{$_} = undef; }
-	
-	foreach $_ (@tasks) {
-	  
-	    if (my $task = &match_task($_)) {
-		my $model = $task->{'model'};
-		my $object = $task->{'list'};
-		if ($object eq $list->{'name'}) { $used_list_models{$model} = 1; }
+	    my %used_list_models; # stores which models already have a task 
+	    foreach (@list_models) { $used_list_models{$_} = undef; }
+	    
+	    foreach my $model (&Task::get_used_models($list->get_list_id())) {
+		$used_list_models{$model} = 1; 
 	    }
-	}
-
-	foreach my $model (keys %used_list_models) {
-	    unless ($used_list_models{$model}) {
-		my $model_task_parameter = "$model".'_task';
-		
-		if ( $model eq 'sync_include') {
-		    next unless (($list->{'admin'}{'user_data_source'} eq 'include2') &&
-				 $list->has_include_data_sources() &&
-				 ($list->{'admin'}{'status'} eq 'open'));
+	    
+	    foreach my $model (@list_models) {
+		unless ($used_list_models{$model}) {
+		    my $model_task_parameter = "$model".'_task';
 		    
-		    create ($current_date, 'INIT', $model, 'ttl', 'list', \%data);
-
-		}elsif ($list->{'admin'}{$model_task_parameter} ) {
-		    create ($current_date, '', $model, $list->{'admin'}{$model_task_parameter}{'name'}, 
-			    'list', \%data);
+		    if ( $model eq 'sync_include') {
+			next unless (($list->{'admin'}{'user_data_source'} eq 'include2') &&
+				     $list->has_include_data_sources() &&
+				     ($list->{'admin'}{'status'} eq 'open'));
+			
+			create ($current_date, 'INIT', $model, 'ttl', \%data);
+			
+		    }elsif (defined $list->{'admin'}{$model_task_parameter} && 
+			    defined $list->{'admin'}{$model_task_parameter}{'name'}) {
+			
+			create ($current_date, '', $model, $list->{'admin'}{$model_task_parameter}{'name'}, \%data);
+		    }
 		}
 	    }
 	}
@@ -309,30 +313,65 @@ while (!$end) {
     my $rep = &tools::adate ($current_date);
 
     ## Execute existing tasks
-    unless (opendir(DIR, $spool_task)) {
-	&do_log ('err', "error : can't open dir %s: %m", $spool_task);
+    ## List all tasks
+    unless (&Task::list_tasks($spool_task)) {
+	&List::send_notify_to_listmaster('intern_error',$Conf{'domain'},{'error' => "Failed to list task files in $spool_task"})
+	&do_log ('err', "Failed to list task files in %s", $spool_task);
+	exit -1;
     }
-    my @tasks = sort epoch_sort (grep !/^\.\.?$/, readdir DIR);
 
     ## processing of tasks anterior to the current date
     &do_log ('debug3', 'processing of tasks anterior to the current date');
-    foreach (@tasks) {
-	if (my $task = &match_task($_)) {
-	    &do_log ('debug3', "procesing %s/%s", $spool_task,$_);
-	    last unless ($task->{'date'} < $current_date);
-	    if ($task->{'list'} ne '_global') { # list task
-		my $list = new List ($task->{'list'});
-		next unless ($list->{'admin'}{'status'} eq 'open');
+    foreach my $task ( &Task::get_task_list() ) {
+	my $task_file = $task->{'filepath'};
+
+	&do_log ('debug3', "procesing %s", $task_file);
+	last unless ($task->{'date'} < $current_date);
+	if ($task->{'object'} ne '_global') { # list task
+	    my $list = $task->{'list_object'};
+	    
+	    ## Skip closed lists
+	    unless (defined $list && ($list->{'admin'}{'status'} eq 'open')) {
+		&do_log('notice','Removing task file %s', $task_file);
+		unless (unlink $task_file) {
+		    &do_log('err', 'Unable to remove task file %s : %s', $task_file, $!);
+		    next;
+		}
+		next;
+	    }
+	    
+	    ## Skip if parameter is not defined
+	    if ( $task->{'model'} eq 'sync_include') {
+		unless (($list->{'admin'}{'user_data_source'} eq 'include2') &&
+			$list->has_include_data_sources() &&
+			($list->{'admin'}{'status'} eq 'open')) {
+		    &do_log('notice','Removing task file %s', $task_file);
+		    unless (unlink $task_file) {
+			&do_log('err', 'Unable to remove task file %s : %s', $task_file, $!);
+			next;
+		    }
+		    next;
+		}
+	    }else {
+		unless (defined $list->{'admin'}{$task->{'model'}} && 
+			defined $list->{'admin'}{$task->{'model'}}{'name'}) {
+		    &do_log('notice','Removing task file %s', $task_file);
+		    unless (unlink $task_file) {
+			&do_log('err', 'Unable to remove task file %s : %s', $task_file, $!);
+			next;
+		    }
+		    next;
+		}		
 	    }
-	    execute ("$spool_task/$_");
 	}
+	execute ($task);
     }
 
     sleep 60;
     #$end = 1;
 
     ## Free zombie sendmail processes
-    &smtp::reaper;
+    &mail::reaper;
 }
 
 &do_log ('notice', 'task_manager exited normally due to signal'); 
@@ -350,18 +389,24 @@ sub create {
     my $label         = shift;
     my $model         = shift;
     my $model_choice  = shift;
-    my $object        = shift;
     my $Rdata         = shift;
 
-    &do_log ('debug2', "create date : $date label : $label model $model : $model_choice object : $object Rdata :$Rdata");
+    &do_log ('debug2', "create date : $date label : $label model $model : $model_choice Rdata :$Rdata");
 
     my $task_file;
     my $list_name;
-    if ($object eq 'list') { 
+    my $robot;
+    my $object;
+    if (defined $Rdata->{'list'}) { 
 	$list_name = $Rdata->{'list'}{'name'};
-	$task_file  = "$spool_task/$date.$label.$model.$list_name";
+	$robot = $Rdata->{'list'}{'robot'};
+	$task_file  = "$spool_task/$date.$label.$model.$list_name\@$robot";
+	$object = 'list';
+    }
+    else {
+	$object = '_global';
+	$task_file  = $spool_task.'/'.$date.'.'.$label.'.'.$model.'.'.$object;
     }
-    else {$task_file  = $spool_task.'/'.$date.'.'.$label.'.'.$model.'.'.$object;}
 
     ## model recovery
     my $model_file;
@@ -379,7 +424,7 @@ sub create {
 
     # for a list
     if ($object  eq 'list') {
-	my $list = new List($list_name);
+	my $list = new List($list_name, $robot);
 
 	$Rdata->{'list'}{'ttl'} = $list->{'admin'}{'ttl'};
 
@@ -401,7 +446,7 @@ sub create {
     # special checking for list whose user_data_source config parmater is include. The task won't be created if there is a delete_subs command
     my $ok = 1;
     if ($object eq 'list') {
-	my $list = new List("$list_name");
+	my $list = new List($list_name, $robot);
 	if ($list->{'admin'}{'user_data_source'} eq 'include') {
 	    unless ( open (TASK, $task_file) ) {
 		&do_log ('err', "error : unable to read $task_file, checking is impossible");
@@ -518,6 +563,9 @@ sub chk_line {
     my $line = $_[0];
     my $Rhash = $_[1]; # will contain nature of line (label, command, error...)
 
+    ## just in case...
+    chomp $line;
+
     &do_log('debug2', 'chk_line(%s, %s)', $line, $Rhash->{'nature'});
         
     $Rhash->{'nature'} = undef;
@@ -652,30 +700,27 @@ sub chk_cmd {
 
 sub execute {
 
-    my $task_file = $_[0]; # task to execute
+    my $task = shift;
+    my $task_file = $task->{'filepath'}; # task to execute
+
     my %result; # stores the result of the chk_line subroutine
     my %vars; # list of task vars
     my $lnb = 0; # line number
 
-    &do_log('debug2', 'execute(%s, %d, %s)', $task_file, $lnb, join('/',  %vars));
+    &do_log('notice', 'Running task %s, line %d with vars %s)', $task_file, $lnb, join('/',  %vars));
     
     unless ( open (TASK, $task_file) ) {
 	&do_log ('err', "error : can't read the task $task_file");
 	return undef;
     }
 
-    # get the task name, without the path
-    my @path = split /\\/, $task_file;
-    my $task_name = $path[$#path];
-
-    # positioning at the right label
-    $task_name =~ /\w*\.(\w*)\..*/;
-    my $label = $1;
+    my $label = $task->{'label'};
     return undef if ($label eq 'ERROR');
 
     &do_log ('debug2', "* execution of the task $task_file");
     unless ($label eq '') {
 	while ( <TASK> ) {
+	    chomp;
 	    $lnb++;
 	    chk_line ($_, \%result);
 	    last if ($result{'label'} eq $label);
@@ -696,13 +741,13 @@ sub execute {
 	
 	# processing of the assignments
 	if ($result{'nature'} eq 'assignment') {
-	    $status = $vars{$result{'var'}} = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task_file, \%vars, $lnb);
+	    $status = $vars{$result{'var'}} = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task, \%vars, $lnb);
 	    last unless defined($status);
 	}
 	
 	# processing of the commands
 	if ($result{'nature'} eq 'command') {
-	    $status = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task_file, \%vars, $lnb);
+	    $status = &cmd_process ($result{'command'}, $result{'Rarguments'}, $task, \%vars, $lnb);
 	    last unless defined($status);
 	}
     } 
@@ -726,46 +771,42 @@ sub cmd_process {
 
     my $command = $_[0]; # command name
     my $Rarguments = $_[1]; # command arguments
-    my $task_file = $_[2]; # task
+    my $task = $_[2]; # task
     my $Rvars = $_[3]; # variable list of the task
     my $lnb = $_[4]; # line number
 
+    my $task_file = $task->{'filepath'};
+
     &do_log('debug2', 'cmd_process(%s, %s, %d)', $command, $task_file, $lnb);
 
      # building of %context
-    my %context; # datas necessary to command processing
-    $context{'task_file'} = $task_file; # long task file name
-    $task_file =~ /\/($tools::regexp{'listname'})$/i;
-    $context{'task_name'} = $1; # task file name
-    $context{'task_name'} =~ /^(\d+)\..+/;
-    $context{'execution_date'} = $1; # task execution date
-    $context{'task_name'} =~ /^\w+\.\w*\.\w+\.($tools::regexp{'listname'})$/;
-    $context{'object_name'} = $1; # object of the task
-    $context{'line_number'} = $lnb;
+    my %context = ('line_number' => $lnb);
+
+    &do_log('debug2','Current task : %s', join(':',%$task));
 
      # regular commands
-    return stop (\%context) if ($command eq 'stop');
-    return next_cmd ($Rarguments, \%context) if ($command eq 'next');
-    return create_cmd ($Rarguments, \%context) if ($command eq 'create');
-    return exec_cmd ($Rarguments) if ($command eq 'exec');
-    return update_crl ($Rarguments, \%context) if ($command eq 'update_crl');
-    return expire_bounce ($Rarguments, \%context) if ($command eq 'expire_bounce');
-    return purge_user_table (\%context) if ($command eq 'purge_user_table');
-    return sync_include(\%context) if ($command eq 'sync_include');
-    return purge_orphan_bounces (\%context) if ($command eq 'purge_orphan_bounces');
-    return eval_bouncers (\%context) if ($command eq 'eval_bouncers');
-    return process_bouncers (\%context) if ($command eq 'process_bouncers');
+    return stop ($task, \%context) if ($command eq 'stop');
+    return next_cmd ($task, $Rarguments, \%context) if ($command eq 'next');
+    return create_cmd ($task, $Rarguments, \%context) if ($command eq 'create');
+    return exec_cmd ($task, $Rarguments) if ($command eq 'exec');
+    return update_crl ($task, $Rarguments, \%context) if ($command eq 'update_crl');
+    return expire_bounce ($task, $Rarguments, \%context) if ($command eq 'expire_bounce');
+    return purge_user_table ($task, \%context) if ($command eq 'purge_user_table');
+    return sync_include($task, \%context) if ($command eq 'sync_include');
+    return purge_orphan_bounces ($task, \%context) if ($command eq 'purge_orphan_bounces');
+    return eval_bouncers ($task, \%context) if ($command eq 'eval_bouncers');
+    return process_bouncers ($task, \%context) if ($command eq 'process_bouncers');
 
      # commands which use a variable
-    return send_msg ($Rarguments, $Rvars, \%context) if ($command eq 'send_msg');       
-    return rm_file ($Rarguments, $Rvars, \%context) if ($command eq 'rm_file');
+    return send_msg ($task, $Rarguments, $Rvars, \%context) if ($command eq 'send_msg');       
+    return rm_file ($task, $Rarguments, $Rvars, \%context) if ($command eq 'rm_file');
 
      # commands which return a variable
-    return select_subs ($Rarguments, \%context) if ($command eq 'select_subs');
-    return chk_cert_expiration ($Rarguments, \%context) if ($command eq 'chk_cert_expiration');
+    return select_subs ($task, $Rarguments, \%context) if ($command eq 'select_subs');
+    return chk_cert_expiration ($task, $Rarguments, \%context) if ($command eq 'chk_cert_expiration');
 
      # commands which return and use a variable
-    return delete_subs_cmd ($Rarguments, $Rvars, \%context) if ($command eq 'delete_subs');  
+    return delete_subs_cmd ($task, $Rarguments, $Rvars, \%context) if ($command eq 'delete_subs');  
 }
 
 
@@ -774,9 +815,7 @@ sub cmd_process {
  # remove files whose name is given in the key 'file' of the hash
 sub rm_file {
     
-    my $Rarguments = $_[0];
-    my $Rvars = $_[1];
-    my $context = $_[2];
+    my ($task, $Rarguments,$Rvars, $context) = @_;
     
     my @tab = @{$Rarguments};
     my $var = $tab[0];
@@ -785,7 +824,7 @@ sub rm_file {
 	my $file = $Rvars->{$var}{$key}{'file'};
 	next unless ($file);
 	unless (unlink ($file)) {
-	    error ("$context->{'task_file'}", "error in rm_file command : unable to remove $file");
+	    error ($task->{'filepath'}, "error in rm_file command : unable to remove $file");
 	    return undef;
 	}
     }
@@ -794,9 +833,9 @@ sub rm_file {
 }
 
 sub stop {
-
-    my $context = $_[0];
-    my $task_file = $context->{'task_file'};
+    
+    my ($task, $context) = @_;
+    my $task_file = $spool_task.'/'.$task->{'file'};
 
     &do_log ('notice', "$context->{'line_number'} : stop $task_file");
     
@@ -809,9 +848,7 @@ sub stop {
 
 sub send_msg {
         
-    my $Rarguments = $_[0];
-    my $Rvars = $_[1];
-    my $context = $_[2];
+    my ($task, $Rarguments, $Rvars, $context) = @_;
     
     my @tab = @{$Rarguments};
     my $template = $tab[1];
@@ -820,45 +857,50 @@ sub send_msg {
     &do_log ('notice', "line $context->{'line_number'} : send_msg (@{$Rarguments})");
 
 
-    if ($context->{'object_name'} eq '_global') {
+    if ($task->{'object'} eq '_global') {
 
 	foreach my $email (keys %{$Rvars->{$var}}) {
 	    &do_log ('notice', "--> message sent to $email");
-	    &List::send_global_file ($template, $email, $Rvars->{$var}{$email}) if (!$log);
+ 	    if (!$log) {
+ 		unless (&List::send_global_file ($template, $email, ,'',$Rvars->{$var}{$email}) ) {
+ 		    &do_log ('notice', "Unable to send template $template to $email");
+ 		}
+ 	    }
 	}
     } else {
-	my $list = new List ($context->{'object_name'});
-        
+	my $list = $task->{'list_object'};
+	
 	foreach my $email (keys %{$Rvars->{$var}}) {
 	    &do_log ('notice', "--> message sent to $email");
-	    $list->send_file ($template, $email, $Rvars->{$var}{$email}) if (!$log);
+ 	    if (!$log) {
+ 		unless ($list->send_file ($template, $email, $list->{'domain'}, $Rvars->{$var}{$email}))  {
+ 		    &do_log ('notice', "Unable to send template $template to $email");
+ 		}
+	    }
 	}
     }
     return 1;
 }
 
 sub next_cmd {
-    
-    my $Rarguments = $_[0];
-    my $context = $_[1];
+        
+    my ($task, $Rarguments, $context) = @_;
     
     my @tab = @{$Rarguments};
-    my $date = &tools::epoch_conv ($tab[0], $context->{'execution_date'}); # conversion of the date argument into epoch format
+    my $date = &tools::epoch_conv ($tab[0], $task->{'date'}); # conversion of the date argument into epoch format
     my $label = $tab[1];
 
-    &do_log ('notice', "line $context->{'line_number'} of $context->{'task_name'} : next ($date, $label)");
+    &do_log ('notice', "line $context->{'line_number'} of $task->{'model'} : next ($date, $label)");
 
-    my @name = split /\./, $context->{'task_name'};
-
-    ## Last item (listname) can contain '.' chars
-    $name[3] = join('.',@name[3..$#name]);
-    my $model = $name[2];
+    my $listname = $task->{'object'};
+    my $model = $task->{'model'};
+    my $filename = $task->{'filepath'};
 
     ## Determine type
     my ($type, $model_choice);
-    my %data = ('creation_date'  => $context->{'execution_date'},
+    my %data = ('creation_date'  => $task->{'date'},
 		'execution_date' => 'execution_date');
-    if ($name[3] eq '_global') {
+    if ($listname eq '_global') {
 	$type = '_global';
 	foreach my $key (keys %global_models) {
 	    if ($global_models{$key} eq $model) {
@@ -868,12 +910,13 @@ sub next_cmd {
 	}
     }else {
 	$type = 'list';
-	my $list = new List($name[3]);
+	my $list = $task->{'list_object'};
 	$data{'list'}{'name'} = $list->{'name'};
+	$data{'list'}{'robot'} = $list->{'domain'};
 	
 	if ( $model eq 'sync_include') {
 	    unless ($list->{'admin'}{'user_data_source'} eq 'include2') {
-		error ($context->{'task_file'}, "List $list->{'name'} no more require sync_include task");
+		error ($filename, "List $list->{'name'} no more require sync_include task");
 		return undef;
 	    }
 
@@ -881,7 +924,7 @@ sub next_cmd {
 	    $model_choice = 'ttl';
 	}else {
 	    unless (defined $list->{'admin'}{"$model\_task"}) {
-		error ($context->{'task_file'}, "List $list->{'name'} no more require $model task");
+		error ($filename, "List $list->{'name'} no more require $model task");
 		return undef;
 	    }
 
@@ -889,20 +932,20 @@ sub next_cmd {
 	}
     }
 
-    unless (create ($date, $tab[1], $name[2], $model_choice, $type, \%data)) {
-	error ($context->{'task_file'}, "error in create command : creation subroutine failure");
+    unless (create ($date, $tab[1], $model, $model_choice, \%data)) {
+	error ($filename, "error in create command : creation subroutine failure");
 	return undef;
     }
 
 #    my $new_task = "$date.$label.$name[2].$name[3]";
     my $human_date = &tools::adate ($date);
 #    my $new_task_file = "$spool_task/$new_task";
-#    unless (rename ($context->{'task_file'}, $new_task_file)) {
-#	error ("$context->{'task_file'}", "error in next command : unable to rename task file into $new_task");
+#    unless (rename ($filename, $new_task_file)) {
+#	error ($filename, "error in next command : unable to rename task file into $new_task");
 #	return undef;
 #    }
-    unless (unlink ($context->{'task_file'})) {
-	error ("$context->{'task_file'}", "error in next command : unable to remove task file $context->{'task_file'}");
+    unless (unlink ($filename)) {
+	error ($filename, "error in next command : unable to remove task file $filename");
 	return undef;
     }
 
@@ -913,8 +956,7 @@ sub next_cmd {
 
 sub select_subs {
 
-    my $Rarguments = $_[0];
-    my $context = $_[1];
+    my ($task, $Rarguments, $context) = @_;
 
     my @tab = @{$Rarguments};
     my $condition = $tab[0];
@@ -922,13 +964,13 @@ sub select_subs {
     &do_log ('debug2', "line $context->{'line_number'} : select_subs ($condition)");
     $condition =~ /(\w+)\(([^\)]*)\)/;
     if ($2) { # conversion of the date argument into epoch format
-	my $date = &tools::epoch_conv ($2, $context->{'execution_date'});
+	my $date = &tools::epoch_conv ($2, $task->{'date'});
         $condition = "$1($date)";
     }  
  
     my @users; # the subscribers of the list      
     my %selection; # hash of subscribers who match the condition
-    my $list = new List ($context->{'object_name'});
+    my $list = $task->{'list_object'};
     
     if ( $list->{'admin'}{'user_data_source'} =~ /database|file|include2/) {
         for ( my $user = $list->get_first_user(); $user; $user = $list->get_next_user() ) { 
@@ -940,7 +982,7 @@ sub select_subs {
     my $verify_context = {'sender' => 'nobody',
 			  'email' => 'nobody',
 			  'remote_host' => 'unknown_host',
-			  'listname' => $context->{'object_name'}};
+			  'listname' => $task->{'object'}};
     
     my $new_condition = $condition; # necessary to the older & newer condition rewriting
     # loop on the subscribers of $list_name
@@ -961,9 +1003,7 @@ sub select_subs {
 
 sub delete_subs_cmd {
 
-    my $Rarguments = $_[0];
-    my $Rvars = $_[1];
-    my $context = $_[2];
+    my ($task, $Rarguments, $Rvars,  $context) = @_;
 
     my @tab = @{$Rarguments};
     my $var = $tab[0];
@@ -971,19 +1011,20 @@ sub delete_subs_cmd {
     &do_log ('notice', "line $context->{'line_number'} : delete_subs ($var)");
 
     
-    my $list = new List ($context->{'list_name'});
+    my $list = $task->{'list_object'};
     my %selection; # hash of subscriber emails who are successfully deleted
 
     foreach my $email (keys %{$Rvars->{$var}}) {
 
 	&do_log ('notice', "email : $email");
-	my $action = &List::request_action ('del', 'smime',
-					    {'listname' => $context->{'list_name'},
-					     'sender'   => $Conf{'listmaster'},
-					     'email'    => $email,
-					 });
+	my $result = $list->check_list_authz('del', 'smime',
+					     {'sender'   => $Conf{'listmaster'},
+					      'email'    => $email,
+					  });
+	my $action;
+	$action = $result->{'action'} if (ref($result) eq 'HASH');
 	if ($action =~ /reject/i) {
-	    error ("$context->{'task_file'}", "error in delete_subs command : deletion of $email not allowed");
+	    error ($task->{'filepath'}, "error in delete_subs command : deletion of $email not allowed");
 	} else {
 	    my $u = $list->delete_user ($email) if (!$log);
 	    $list->save() if (!$log);;
@@ -997,8 +1038,7 @@ sub delete_subs_cmd {
 
 sub create_cmd {
 
-    my $Rarguments = $_[0];
-    my $context = $_[1];
+    my ($task, $Rarguments, $context) = @_;
 
     my @tab = @{$Rarguments};
     my $arg = $tab[0];
@@ -1014,12 +1054,12 @@ sub create_cmd {
 	$type = $1;
 	$object = $3;
     } else {
-	error ($context->{'task_file'}, "error in create command : don't know how to create $arg");
+	error ($task->{'filepath'}, "error in create command : don't know how to create $arg");
 	return undef;
     }
 
     # building of the data hash necessary to the create subroutine
-    my %data = ('creation_date'  => $context->{'execution_date'},
+    my %data = ('creation_date'  => $task->{'date'},
 		'execution_date' => 'execution_date');
 
     if ($type eq 'list') {
@@ -1028,8 +1068,8 @@ sub create_cmd {
     }
     $type = '_global';
     #printf "xxxxxxxxxxxxx appel 3\n";
-    unless (create ($context->{'execution_date'}, '', $model, $model_choice, $type, \%data)) {
-	error ($context->{'task_file'}, "error in create command : creation subroutine failure");
+    unless (create ($task->{'date'}, '', $model, $model_choice, \%data)) {
+	error ($task->{'filepath'}, "error in create command : creation subroutine failure");
 	return undef;
     }
     
@@ -1038,8 +1078,7 @@ sub create_cmd {
 
 sub exec_cmd {
 
-    my $Rarguments = $_[0];
-    my $context = $_[1];
+    my ($task, $Rarguments, $context) = @_;
 
     my @tab = @{$Rarguments};
     my $file = $tab[0];
@@ -1051,8 +1090,7 @@ sub exec_cmd {
 }
 
 sub purge_user_table {
-    my $Rarguments = $_[0];
-    my $context = $_[1];
+    my ($task, $Rarguments, $context) = @_;
     do_log('debug2','purge_user_table()');
 
     ## Load user_table entries
@@ -1067,18 +1105,24 @@ sub purge_user_table {
     }
 
     foreach my $r (keys %{$Conf{'robots'}}) {
-	foreach my $l (&List::get_lists($r)){
-	    my $list = new List($l);
-	    next unless defined($list);
+
+	my $all_lists = &List::get_lists($r);
+	foreach my $list (@$all_lists){
 
 	    ## Owners
-	    foreach my $o (@{$list->{'admin'}{'owner'}}) {
-		$known_people{$o->{'email'}} = 1;
+	    my $owners = $list->get_owners();
+	    if (defined $owners) {
+		foreach my $o (@{$owners}) {
+		    $known_people{$o->{'email'}} = 1;
+		}
 	    }
 
 	    ## Editors
-	    foreach my $e (@{$list->{'admin'}{'editor'}}) {
-		$known_people{$e->{'email'}} = 1;
+	    my $editors = $list->get_editors();
+	    if (defined $editors) {		
+		foreach my $e (@{$editors}) {
+		    $known_people{$e->{'email'}} = 1;
+		}
 	    }
 	    
 	    ## Subscribers
@@ -1107,24 +1151,25 @@ sub purge_user_table {
     return $#purged_users + 1;
 }
 
-sub purge_orphan_bounces {
 ## Subroutine which remove bounced message of no-more known users
+sub purge_orphan_bounces {
+    my($task, $context) = @_;
     
     do_log('info','purge_orphan_bounces()');
 
     ## Hash {'listname' => 'bounced address' => 1}
     my %bounced_users;
-    my @listes;
+    my $all_lists;
 
-    unless (@listes = &List::get_lists('*')) {
-	&do_log('err','error while loading list of listes with function get_lists');
+    unless ($all_lists = &List::get_lists('*')) {
+	&do_log('notice','No list available');
+	return 1;
     }
 
 
-    foreach my $listname (@listes) {
-	 my $list = new List ($listname);	
+    foreach my $list (@$all_lists) {
 
-	 next unless (defined $list);
+	my $listname = $list->{'name'};
 
 	 ## first time: loading DB entries into %bounced_users
 	 for (my $user_ref = $list->get_first_bouncing_user(); $user_ref; $user_ref = $list->get_next_bouncing_user()){
@@ -1132,14 +1177,16 @@ sub purge_orphan_bounces {
 	     $bounced_users{$listname}{$user_id} = 1;
 	 }
 
-	 unless (-d $wwsconf->{'bounce_path'}.'/'.$listname) {
+	my $bounce_dir = $list->get_bounce_dir();
+
+	 unless (-d $bounce_dir) {
 	     &do_log('notice', 'No bouncing subscribers in list %s', $listname);
 	     next;
 	 }
 
 	 ## then reading Bounce directory & compare with %bounced_users
-	 unless (opendir(BOUNCE,$wwsconf->{'bounce_path'}.'/'.$listname)) {
-	     &do_log('err','Error while opening bounce directory %s for list %s',$wwsconf->{'bounce_path'},$listname);
+	 unless (opendir(BOUNCE,$bounce_dir)) {
+	     &do_log('err','Error while opening bounce directory %s',$bounce_dir);
 	     return undef;
 	 }
 
@@ -1148,12 +1195,14 @@ sub purge_orphan_bounces {
 	     if ($bounce =~ /\@/){
 		 unless (defined($bounced_users{$listname}{$bounce})) {
 		     &do_log('info','removing orphan Bounce for user %s in list %s',$bounce,$listname);
-		     unless (unlink($wwsconf->{'bounce_path'}.'/'.$listname.'/'.$bounce)) {
-			 &do_log('err','Error while removing file %s',$bounce);
+		     unless (unlink($bounce_dir.'/'.$bounce)) {
+			 &do_log('err','Error while removing file %s/%s', $bounce_dir, $bounce);
 		     }
 		 }
 	     }	    
 	 }
+
+	closedir BOUNCE;
      }
  }
 
@@ -1161,18 +1210,17 @@ sub purge_orphan_bounces {
  sub expire_bounce {
      # If a bounce is older then $list->get_latest_distribution_date()-$delai expire the bounce
      # Is this variable my be set in to task modele ?
-     my $Rarguments = $_[0];
-     my $context = $_[1];
+     my ($task, $Rarguments, $context) = @_;
 
-     my $execution_date = $context->{'execution_date'};
+     my $execution_date = $task->{'date'};
      my @tab = @{$Rarguments};
      my $delay = $tab[0];
 
      do_log('debug2','expire_bounce(%d)',$delay);
-     foreach my $listname (&List::get_lists('*') ) {
-	 my $list = new List ($listname);
+     my $all_lists = &List::get_lists('*');
+     foreach my $list (@$all_lists ) {
 
-	 next unless $list;
+	 my $listname = $list->{'name'};
 
 	 # the reference date is the date until which we expire bounces in second
 	 # the latest_distribution_date is the date of last distribution #days from 01 01 1970
@@ -1198,14 +1246,17 @@ sub purge_orphan_bounces {
 		     next;
 		 }
 
-		 unless( $list->update_user($email, {'bounce' => 'NULL'})) {
+		 unless( $list->update_user($email, {'bounce' => 'NULL'},{'bounce_address' => 'NULL'})) {
 		     do_log('info','expire_bounce: failed update database for %s', $email);
 		     next;
 		 }
 		 my $escaped_email = &tools::escape_chars($email);
-		 unless (unlink "$wwsconf->{'bounce_path'}/$listname/$escaped_email") {
-		     do_log('info','expire_bounce: failed deleting %s', "$wwsconf->{'bounce_path'}/$listname/$escaped_email");
-		    next;
+
+		 my $bounce_dir = $list->get_bounce_dir();
+
+		 unless (unlink $bounce_dir.'/'.$escaped_email) {
+		     &do_log('info','expire_bounce: failed deleting %s', $bounce_dir.'/'.$escaped_email);
+		     next;
 		 }
 		 do_log('info','expire bounces for subscriber %s of list %s (last distribution %s, last bounce %s )',
 			$email,$listname,
@@ -1221,10 +1272,9 @@ sub purge_orphan_bounces {
 
  sub chk_cert_expiration {
 
-     my $Rarguments = $_[0];
-     my $context = $_[1];
+     my ($task, $Rarguments, $context) = @_;
 
-     my $execution_date = $context->{'execution_date'};
+     my $execution_date = $task->{'date'};
      my @tab = @{$Rarguments};
      my $template = $tab[0];
      my $limit = &tools::duration_conv ($tab[1], $execution_date);
@@ -1233,11 +1283,11 @@ sub purge_orphan_bounces {
 
      ## building of certificate list
      unless (opendir(DIR, $cert_dir)) {
-	 error ($context->{'task_file'}, "error in chk_cert_expiration command : can't open dir $cert_dir");
+	 error ($task->{'filepath'}, "error in chk_cert_expiration command : can't open dir $cert_dir");
 	 return undef;
      }
      my @certificates = grep !/^(\.\.?)|(.+expired)$/, readdir DIR;
-     close (DIR);
+     closedir (DIR);
 
      foreach (@certificates) {
 
@@ -1309,7 +1359,12 @@ sub purge_orphan_bounces {
 	     $tpl_context{'expiration_date'} = &tools::adate ($expiration_date);
 	     $tpl_context{'certificate_id'} = $id;
 
-	     &List::send_global_file ($template, $_, \%tpl_context) if (!$log);
+	     
+	     if (!$log) {
+		 unless (&List::send_global_file ($template, $_,'', \%tpl_context)) {
+		     &do_log ('notice', "Unable to send template $template to $_");
+		 }
+	     }
 	     &do_log ('notice', "--> $_ certificate soon expired ($date), user warned");
 	 }
      }
@@ -1320,18 +1375,17 @@ sub purge_orphan_bounces {
  ## attention, j'ai n'ai pas pu comprendre les retours d'erreurs des commandes wget donc pas de verif sur le bon fonctionnement de cette commande
  sub update_crl {
 
-     my $Rarguments = $_[0];
-     my $context = $_[1];
+     my ($task, $Rarguments, $context) = @_;
 
      my @tab = @{$Rarguments};
-     my $limit = &tools::epoch_conv ($tab[1], $context->{'execution_date'});
+     my $limit = &tools::epoch_conv ($tab[1], $task->{'date'});
      my $CA_file = "$Conf{'home'}/$tab[0]"; # file where CA urls are stored ;
      &do_log ('notice', "line $context->{'line_number'} : update_crl (@tab)");
 
      # building of CA list
      my @CA;
      unless (open (FILE, $CA_file)) {
-	 error ($context->{'task_file'}, "error in update_crl command : can't open $CA_file file");
+	 error ($task->{'filepath'}, "error in update_crl command : can't open $CA_file file");
 	 return undef;
      }
      while (<FILE>) {
@@ -1380,7 +1434,7 @@ sub purge_orphan_bounces {
 	 my $rep = &tools::adate ($expiration_date);
 
 	 ## check if the crl is soon expired or expired
-	 #my $file_date = $context->{'execution_date'} - (-M $file) * 24 * 60 * 60; # last modification date
+	 #my $file_date = $task->{'date'} - (-M $file) * 24 * 60 * 60; # last modification date
 	 my $condition = "newer($limit, $expiration_date)";
 	 my $verify_context;
 	 $verify_context->{'sender'} = 'nobody';
@@ -1401,17 +1455,19 @@ sub purge_orphan_bounces {
  # give a score for each bouncing user
  sub eval_bouncers {
  #################       
+     my ($task, $context) = @_;
 
-     foreach my $listname (&List::get_lists()) {
+     my $all_lists = &List::get_lists('*');
+     foreach my $list (@$all_lists) {
 
-	 my $list = new List ($listname);
+	 my $listname = $list->{'name'};
 	 my $list_traffic = {};
 
 	 &do_log('info','eval_bouncers(%s)',$listname);
 
 	 ## Analizing file Msg-count and fill %$list_traffic
 	 unless (open(COUNT,$list->{'dir'}.'/msg_count')){
-	     &do_log('err','** ERROR WHILE OPENING msg_count FILE for liste %s',$listname);
+	     &do_log('debug','** Could not open msg_count FILE for list %s',$listname);
 	     next;
 	 }    
 	 while (<COUNT>) {
@@ -1444,7 +1500,7 @@ sub none {
 ##
 sub process_bouncers {
 ###################
-
+    my ($task,$context) = @_;
     &do_log('info','Processing automatic actions on bouncing users'); 
 
 ###########################################################################
@@ -1467,8 +1523,9 @@ sub process_bouncers {
 		   'none'            => \&none
 		   );
 
-    foreach my $listname (&List::get_lists()) {
-	my $list = new List ($listname);
+    my $all_lists = &List::get_lists();
+    foreach my $list (@$all_lists) {
+	my $listname = $list->{'name'};
 	
 	my @bouncers;
 	# @bouncers = ( ['email1', 'email2', 'email3',....,],    There is one line 
@@ -1503,28 +1560,28 @@ sub process_bouncers {
 	    my $action = $list->{'admin'}{'bouncers_level'.$level}{'action'};
 	    my $notification = $list->{'admin'}{'bouncers_level'.$level}{'notification'};
 	  
-	    if (@bouncers[$level]){
+	    if (@{$bouncers[$level]}){
 		## calling action subroutine with (list,email list) in parameter 
-		unless ($actions{$action}->($list,@bouncers[$level])){
+		unless ($actions{$action}->($list,$bouncers[$level])){
 		    &do_log('err','error while calling action sub for bouncing users in list %s',$listname);
 		    return undef;
 		}
 
 		## calling notification subroutine with (list,action, email list) in parameter  
 		
-		my @param = ($listname,$action,@bouncers[$level]);
+		my $param = {'listname' => $listname,
+			     'action' => $action,
+			     'user_list' => \@{$bouncers[$level]},
+			     'total' => $#{$bouncers[$level]} + 1};
 
 	        if ($notification eq 'listmaster'){
 
-		    unless(&List::send_notify_to_listmaster('automatic_bounce_management',$list->{'domain'},@param)){
+		    unless(&List::send_notify_to_listmaster('automatic_bounce_management',$list->{'domain'},$param)){
 			&do_log('err','error while notifying listmaster');
-			return undef;
 		    }
 		}elsif ($notification eq 'owner'){
-		    
-		    unless ($list->new_send_notify_to_owner('automatic_bounce_management',@param)){
+		    unless ($list->send_notify_to_owner('automatic_bounce_management',$param)){
 			&do_log('err','error while notifying owner');
-			return undef;
 		    }
 		}
 	    }
@@ -1594,6 +1651,9 @@ sub get_score {
     }
 
     my $note = $bounce_count * $regularity_rate * $type_rate;
+
+    ## Note should be an integer
+    $note = int($note + 0.5);
 	
 #    $note = 100 if ($note > 100); # shift between message ditrib & bounces => note > 100     
     
@@ -1645,19 +1705,17 @@ sub error {
                  $message";
     do_log ('err', "$message");
     change_label ($task_file, 'ERROR') unless ($task_file eq '');
-    &List::send_notify_to_listmaster ('error in task', $Conf{'domain'}, @param);
+    unless (&List::send_notify_to_listmaster ('error in task', $Conf{'domain'}, \@param)) {
+    	&do_log('notice','error while notifying listmaster about "error_in_task"');
+    }
 }
 
 sub sync_include {
-    my $context = $_[0];
+    my ($task, $context) = @_;
 
-    &do_log('debug2', 'sync_include(%s)', $context->{'object_name'});
+    &do_log('debug2', 'sync_include(%s)', $task->{'id'});
 
-    my $list = new List($context->{'object_name'});
-    unless (defined $list) {
-	error ($context->{'task_file'}, "Unknown list $context->{'object_name'}");
-	return undef;
-    }
+    my $list = $task->{'list_object'};
 
     unless ( $list->{'admin'}{'user_data_source'} eq 'include2' ) {
 	&do_log('notice', 'sync_include() called for %s but user_data_source is %s', 
@@ -1672,22 +1730,6 @@ sub sync_include {
     }    
 
     $list->sync_include();
+    $list->sync_include_admin();
     }
 
-## Check if the provided filename matches a task
-## Returns an array of its parts
-sub match_task {
-    my $filename = shift;
-
-    if ($filename =~ /^(\d+)\.(\w*)\.(\w+)\.($tools::regexp{'listname'}|_global)$/) {
-	my $task = {'date' => $1,
-		    'label' => $2,
-		    'model' => $3,
-		    'list' => $4
-		};
-	return $task;
-    }
-    
-    return undef;
-}
-
