@@ -1,5 +1,7 @@
+#!--PERL--
+
 # tools.pl - This module provides various tools for Sympa
-# RCS Identication ; $Revision: 1.130.2.7 $ ; $Date: 2005/01/27 10:02:26 $ 
+# RCS Identication ; $Revision: 1.181.2.1 $ ; $Date: 2006/10/03 11:15:06 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -32,21 +34,26 @@ use File::Find;
 use Digest::MD5;
 
 ## RCS identification.
-#my $id = '@(#)$Id: tools.pl,v 1.130.2.7 2005/01/27 10:02:26 sympa-authors Exp $';
+#my $id = '@(#)$Id: tools.pl,v 1.181.2.1 2006/10/03 11:15:06 olivier.salaun Exp $';
 
 ## global var to store a CipherSaber object 
 my $cipher;
 
+$separator="------- CUT --- CUT --- CUT --- CUT --- CUT --- CUT --- CUT -------";
+
 ## Regexps for list params
 ## Caution : if this regexp changes (more/less parenthesis), then regexp using it should 
 ## also be changed
 %regexp = ('email' => '([\w\-\_\.\/\+\=\']+|\".*\")\@[\w\-]+(\.[\w\-]+)+',
+	   'family_name' => '[a-z0-9][a-z0-9\-\.\+_]*', 
 	   'host' => '[\w\.\-]+',
 	   'multiple_host_with_port' => '[\w\.\-]+(:\d+)?(,[\w\.\-]+(:\d+)?)*',
 	   'listname' => '[a-z0-9][a-z0-9\-\.\+_]*',
 	   'sql_query' => '(SELECT|select).*',
 	   'scenario' => '[\w,\.\-]+',
-	   'task' => '\w+'
+	   'task' => '\w+',
+	   'datasource' => '[\w-]+',
+	   'uid' => '[\w\-\.\+]+',
 	   );
 
 my %openssl_errors = (1 => 'an error occurred parsing the command options',
@@ -84,8 +91,21 @@ sub safefork {
    ## No return.
 }
 
-## Check for commands in the body of the message. Returns true
-## if there are some commands in it.
+####################################################
+# checkcommand                              
+####################################################
+# Checks for no command in the body of the message.
+# If there are some command in it, it return true 
+# and send a message to $sender
+# 
+# IN : -$msg (+): ref(MIME::Entity) - message to check
+#      -$sender (+): the sender of $msg
+#      -$robot (+) : robot
+#
+# OUT : -1 if there are some command in $msg
+#       -0 else
+#
+###################################################### 
 sub checkcommand {
    my($msg, $sender, $robot) = @_;
    do_log('debug3', 'tools::checkcommand(msg->head->get(subject): %s,%s)',$msg->head->get('Subject'), $sender);
@@ -96,9 +116,9 @@ sub checkcommand {
 
    ## Check for commands in the subject.
    my $subject = $msg->head->get('Subject');
+
    if ($subject) {
        if ($Conf{'misaddressed_commands_regexp'} && ($subject =~ /^$Conf{'misaddressed_commands_regexp'}\b/im)) {
-	   &rejectMessage($msg, $sender,$robot);
 	   return 1;
        }
    }
@@ -107,7 +127,6 @@ sub checkcommand {
 
    foreach $i (@{$msg->body}) {
        if ($Conf{'misaddressed_commands_regexp'} && ($i =~ /^$Conf{'misaddressed_commands_regexp'}\b/im)) {
-	   &rejectMessage($msg, $sender, $robot);
 	   return 1;
        }
 
@@ -117,47 +136,19 @@ sub checkcommand {
    return 0;
 }
 
-sub rejectMessage {
-   my($msg, $sender, $robot) = @_;
-   do_log('debug2', 'tools::rejectMessage(%s)', $sender);
 
-   *REJ = smtp::smtpto(&Conf::get_robot_conf($robot, 'request'), \$sender);
-   print REJ "To: $sender\n";
-   print REJ "Subject: [sympa] " . Msg(5, 2, "Misadressed message ?") . "\n";
-   printf REJ "MIME-Version: %s\n", Msg(12, 1, '1.0');
-   printf REJ "Content-Type: text/plain; charset=%s\n", Msg(12, 2, 'us-ascii');
-   printf REJ "Content-Transfer-Encoding: %s\n", Msg(12, 3, '7bit');
-   print REJ "\n";
-   printf REJ Msg(5, 3, "\
-Your message has been sent to a list but it seems it contains commands like
-subscribe, signoff, help, index, get, ...
-
-If your message did really contain a command, please note that such messages
-must be sent to %s only.
-
-If it happens that your message was by mistake considered as containing
-commands, then please contact the manager of this service %s
-so that he can take care of your problem.
-
-Thank you for your attention.
-
------- Beginning of the suspect message --------
-"), &Conf::get_robot_conf($robot, 'sympa'), &Conf::get_robot_conf($robot, 'request');
-   $msg->print(\*REJ);
-   print REJ Msg(5, 4, "------- Fin message suspect ---------\n");
-   close(REJ);
-}
 
 ## return a hash from the edit_list_conf file
 sub load_edit_list_conf {
     my $robot = shift;
+    my $list = shift;
     do_log('debug2', 'tools::load_edit_list_conf (%s)',$robot);
 
     my $file;
     my $conf ;
     
     return undef 
-	unless ($file = &tools::get_filename('etc','edit_list.conf',$robot));
+	unless ($file = &tools::get_filename('etc','edit_list.conf',$robot,$list));
 
     unless (open (FILE, $file)) {
 	&do_log('info','Unable to open config file %s', $file);
@@ -191,7 +182,9 @@ sub load_edit_list_conf {
     }
 
     if ($error_in_conf) {
-	&List::send_notify_to_listmaster('edit_list_error', $robot, $file);
+	unless (&List::send_notify_to_listmaster('edit_list_error', $robot, [$file])) {
+	    &do_log('notice',"Unable to send notify 'edit_list_error' to listmaster");
+	}
     }
     
     close FILE;
@@ -232,93 +225,6 @@ sub load_create_list_conf {
     return $conf;
 }
 
-## Loads a complete translation index
-sub load_index {
-    my ($lang, $file) = @_;
-    &do_log('debug', 'tools::load_index(%s,%s)', $lang, $file);
-
-    unless (open INDEX, $file) {
-	do_log('err', 'Unable to open file %s: %s', $file, $!);
-	return undef;
-    }
-    
-    my %index; 
-    my $current_tpl;
-    while (<INDEX>) {
-	if (/^file\s+(\w+)\s*$/) {
-	    $current_tpl = $1;
-	    next;
-	}elsif (/^\s*$/) {
-	    $current_tpl = undef;
-	}elsif ($current_tpl && /^(\d+)\s+(.+)$/) {
-	    $index{$current_tpl}{$1} = $2;
-	}
-    }
-    close INDEX;
-
-    return \%index;
-}
-
-## Saves a complete translation index into file
-sub save_index {
-    my ($lang, $file, $data) = @_;
-    &do_log('debug', 'tools::save_index(%s,%s)', $lang, $file);
-
-    unless (open INDEX, ">$file") {
-	do_log('err', 'Unable to write to file %s: %s', $file, $!);
-	return undef;
-    }
-
-    foreach my $tpl (sort keys %{$data}) {
-	printf INDEX "file %s\n", $tpl;
-	foreach my $ref (sort  %{$data->{$tpl}}) {
-	    printf INDEX "%d %s\n", $ref, $data->{$tpl}{$ref};
-	}
-	print "\n";
-    }
-    close INDEX;
-
-    return 1;
-}
-
-## Returns a structure representing a template translation
-sub load_translation {
-    my ($tpl, $lang, $robot) = @_;
-
-    my $index_file =  &get_filename('etc', "wws_templates/index.$lang", $robot);
-    my $src_file =  &get_filename('etc', "wws_templates/$tpl.src", $robot);
-
-    my %references;
-
-    unless (open SOURCE, $src_file) {
-	do_log('err', 'Unable to open file %s: %s', $src_file, $!);
-	return undef;
-    }
-    
-    while (<SOURCE>) {
-	s/{ref(\d+):([^}]+)}/&get_ref($1, $2, \%references)/eg;
-    }
-
-    close SOURCE;
-
-    my $index = &load_index($lang, $index_file);
-
-    my %trans;
-    foreach my $i (keys %references) {
-        $trans{$i} = {'default' => &escape_html($references{$i}),
-	 	      'translation' => &escape_html($index->{$tpl}{$i})};
-    }
-    return \%trans;
-}
-
-## Get data from a translated reference
-sub get_ref {
-    my ($ref, $string, $references) = @_;
-
-    $references->{$ref} = $string;
-    
-}
-
 sub _add_topic {
     my ($name, $title) = @_;
     my $topic = {};
@@ -352,8 +258,12 @@ sub get_list_list_tpl {
 
 		$list_templates->{$template}{'path'} = $dir;
 
-		if (-r $dir.'/'.$template.'/comment') {
-		    $list_templates->{$template}{'comment'} = $dir.'/'.$template.'/comment';
+		my $locale = &Language::Lang2Locale( &Language::GetLang());
+		## Look for a comment.tt2 in the appropriate locale first
+		if (-r $dir.'/'.$template.'/'.$locale.'/comment.tt2') {
+		    $list_templates->{$template}{'comment'} = $dir.'/'.$template.'/'.$locale.'/comment.tt2';
+		}elsif (-r $dir.'/'.$template.'/comment.tt2') {
+		    $list_templates->{$template}{'comment'} = $dir.'/'.$template.'/comment.tt2';
 		}
 	    }
 	    closedir(DIR);
@@ -363,20 +273,176 @@ sub get_list_list_tpl {
     return ($list_templates);
 }
 
+#to be used before creating a file in a directory that may not exist already. 
+sub mk_parent_dir {
+    my $file = shift;
+    $file =~ /^(.*)\/([^\/])*$/ ;
+    my $dir = $1;
+    return if (-d $dir);
+    return undef unless (mkdir ($dir, 0755));
+}
+
+# shift file renaming it with date. If count is defined, keep $count file and unlink others
+sub shift_file {
+    my $file = shift;
+    my $count = shift;
+    do_log('debug', "shift_file ($file,$count)");
+
+    unless (-f $file) {
+	do_log('info', "shift_file : unknown file $file");
+	return undef;
+    }
+    
+    my @date = localtime (time);
+    my $file_extention = POSIX::strftime ("%Y:%m:%d:%H:%M:%S", @date);
+    
+    unless (rename ($file,$file.'\.'.$file_extention)) {
+	&do_log('err', "shift_file : Cannot rename file $file to $file.$file_extention" );
+	return undef;
+    }
+    if ($count) {
+	$file =~ /^(.*)\/([^\/])*$/ ;
+	my $dir = $1;
+
+	unless (opendir(DIR, $dir)) {
+	    &do_log('err', "shift_file : Cannot read dir $dir" );
+	    return ($file.'\.'.$file_extention);
+	}
+	my $i = 0 ;
+	foreach my $oldfile (reverse (sort (grep (/^$file\./,readdir(DIR))))) {
+	    $i ++;
+	    if ($count lt $i) {
+		if (unlink ($oldfile)) { 
+		    do_log('info', "shift_file : unlink $oldfile");
+		}else{
+		    do_log('info', "shift_file : unable to unlink $oldfile");
+		}
+	    }
+	}
+    }
+    return ($file.'\.'.$file_extention);
+}
+
+sub get_templates_list {
+
+    my $type = shift;
+    my $robot = shift;
+    my $langdir = shift;
+    my $listdir = shift;
+
+    do_log('debug', "get_templates_list ($type, $robot, $langdir, $listdir)");
+ #   do_log('info', "xxxxxxxxxxxxxxxxxxxxxxxxxxxx get_templates_list ($type, $robot, $langdir, $listdir)");
+    unless (($type == 'web')||($type == 'mail')) {
+	do_log('info', 'get_templates_list () : internal error incorrect parameter');
+    }
+
+    if ($langdir eq 'default') {
+	$langdir = '';
+    }else{
+	$langdir = '/'.$langdir;
+    }
+
+    my $distrib_dir = '--ETCBINDIR--/'.$type.'_tt2';
+    my $site_dir = $Conf{'etc'}.'/'.$type.'_tt2'.$langdir;
+    my $robot_dir = $Conf{'etc'}.'/'.$robot.'/'.$type.'_tt2'.$langdir;
+
+    my @try;
+    push @try, $distrib_dir ;
+    push @try, $site_dir ;
+    push @try, $robot_dir;
+    
+    unless ($listdir) {
+	$listdir .='/'.$type.'_tt2'.$langdir;
+	push @try, $listdir ;
+    }
+	
+    my $i = 0 ;
+    my $tpl;
+
+    foreach my $dir (@try) {
+	next unless opendir (DIR, $dir);
+	foreach my $file ( readdir(DIR)) {	    
+	    next unless ($file =~ /\.tt2$/);
+	    if ($dir eq $distrib_dir){$tpl->{$file}{'distrib'} = $dir.'/'.$file;}
+	    if ($dir eq $site_dir)   {$tpl->{$file}{'site'} =  $dir.'/'.$file;}
+	    if ($dir eq $robot_dir)  {$tpl->{$file}{'robot'} = $dir.'/'.$file;}
+	    if ($dir eq $listdir)    {$tpl->{$file}{'listname'} = $dir.'/'.$file;}
+	}
+	closedir DIR;
+    }
+
+#    open DUMP, ">/tmp/dump";
+#    &tools::dump_var($tpl, 0, \*DUMP);
+#    close DUMP;
+    return ($tpl);
+
+}
+
+# return the path for a specific template
+sub get_template_path {
+
+    my $type = shift;
+    my $robot = shift;
+    my $scope = shift;
+    my $tpl = shift;
+    my $lang = shift;
+    my $listname = shift;
+
+    do_log('info', "get_templates_path ($type,$robot,$scope,$tpl,$lang,$listname)");
+    if ($lang eq 'default') {
+	$lang = '';
+    }else{
+	$lang = '/'.$lang;
+    }
+
+    if ($listname) {
+	chomp ($listname);
+	unless ($namedlist = new List ($listname, $robot)) {
+	    return undef;		
+	}
+    }
+
+    my $listdir = $namedlist->{'dir'} if (defined $namedlist);
+
+    unless (($type == 'web')||($type == 'mail')) {
+	do_log('info', 'get_templates_path () : internal error incorrect parameter');
+    }
+
+    my $distrib_dir = '--ETCBINDIR--/'.$type.'_tt2';
+    my $site_dir = $Conf{'etc'}.'/'.$type.'_tt2'.$lang;
+    my $robot_dir = $Conf{'etc'}.'/'.$robot.'/'.$type.'_tt2'.$lang;
+
+
+    if ($scope eq 'list')  {
+	return $listdir.'/'.$type.'_tt2'.$lang.'/'.$tpl ;
+    }
+    if (($scope eq 'robot')||($scope eq 'list'))  {
+	return $robot_dir.'/'.$tpl;
+    }
+    if (($scope eq 'site')||($scope eq 'robot')||($scope eq 'list')) {
+	return $site_dir.'/'.$tpl;
+    }
+    
+    if (($scope eq 'distrib')||($scope eq 'site')||($scope eq 'robot')||($scope eq 'list')) {
+	return $distrib_dir.'/'.$tpl;
+    }
+}
+
 # input object msg and listname, output signed message object
 sub smime_sign {
     my $in_msg = shift;
     my $list = shift;
-    my $dir = shift;
+    my $robot = shift;
 
     do_log('debug2', 'tools::smime_sign (%s,%s)',$in_msg,$list);
 
-    my $self = new List($list);
+    my $self = new List($list, $robot);
     my($cert, $key) = &smime_find_keys($self->{dir}, 'sign');
-    my $temporary_file = $Conf{'tmpdir'}."/".$list.".".$$ ;    
+    my $temporary_file = $Conf{'tmpdir'}."/".$self->get_list_id().".".$$ ;    
+    my $temporary_pwd = $Conf{'tmpdir'}.'/pass.'.$$;
 
     my ($signed_msg,$pass_option );
-    $pass_option = "-passin file:$Conf{'tmpdir'}/pass.$$" if ($Conf{'key_passwd'} ne '') ;
+    $pass_option = "-passin file:$temporary_pwd" if ($Conf{'key_passwd'} ne '') ;
 
     ## Keep a set of header fields ONLY
     ## OpenSSL only needs content type & encoding to generate a multipart/signed msg
@@ -396,8 +462,8 @@ sub smime_sign {
     close(MSGDUMP);
 
     if ($Conf{'key_passwd'} ne '') {
-	unless ( &POSIX::mkfifo("$Conf{'tmpdir'}/pass.$$",0600)) {
-	    do_log('notice', 'Unable to make fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
+	unless ( &POSIX::mkfifo($temporary_pwd,0600)) {
+	    do_log('notice', 'Unable to make fifo for %s',$temporary_pwd);
 	}
     }
 
@@ -407,13 +473,13 @@ sub smime_sign {
     }
 
     if ($Conf{'key_passwd'} ne '') {
-	unless (open (FIFO,"> $Conf{'tmpdir'}/pass.$$")) {
-	    do_log('notice', 'Unable to open fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
+	unless (open (FIFO,"> $temporary_pwd")) {
+	    do_log('notice', 'Unable to open fifo for %s', $temporary_pwd);
 	}
 
 	print FIFO $Conf{'key_passwd'};
 	close FIFO;
-	unlink ("$Conf{'tmpdir'}/pass.$$");
+	unlink ($temporary_pwd);
     }
 
     my $parser = new MIME::Parser;
@@ -464,12 +530,10 @@ sub smime_sign_check {
     ## first step is the msg signing OK ; /tmp/sympa-smime.$$ is created
     ## to store the signer certificat for step two. I known, that's durty.
 
-
-
     my $temporary_file = "/tmp/smime-sender.".$$ ;
     my $trusted_ca_options = '';
-    $trusted_ca_options = "-CAfile $Conf{'cafile'}" if ($Conf{'cafile'});
-    $trusted_ca_options = "-CApath $Conf{'capath'}" if ($Conf{'capath'});
+    $trusted_ca_options = "-CAfile $Conf{'cafile'} " if ($Conf{'cafile'});
+    $trusted_ca_options .= "-CApath $Conf{'capath'} " if ($Conf{'capath'});
     do_log('debug3', "$Conf{'openssl'} smime -verify  $trusted_ca_options -signer  $temporary_file");
 
     unless (open (MSGDUMP, "| $Conf{'openssl'} smime -verify  $trusted_ca_options -signer $temporary_file > /dev/null")) {
@@ -736,7 +800,7 @@ sub smime_decrypt {
     &do_log('debug2', 'tools::smime_decrypt message msg from %s,%s',$msg->head->get('from'),$list->{'name'});
 
     ## an empty "list" parameter means mail to sympa@, listmaster@...
-    my $dir = $list->{dir};
+    my $dir = $list->{'dir'};
     unless ($dir) {
 	$dir = $Conf{home} . '/sympa';
     }
@@ -746,8 +810,9 @@ sub smime_decrypt {
 	return undef;
     }
 
-    my $temporary_file = $Conf{'tmpdir'}."/".$list->{'name'}.".".$$ ;
-    
+    my $temporary_file = $Conf{'tmpdir'}."/".$list->get_list_id().".".$$ ;
+    my $temporary_pwd = $Conf{'tmpdir'}.'/pass.'.$$;
+
     ## dump the incomming message.
     if (!open(MSGDUMP,"> $temporary_file")) {
 	&do_log('info', 'Can\'t store message in file %s',$temporary_file);
@@ -758,7 +823,7 @@ sub smime_decrypt {
     my ($decryptedmsg, $pass_option, $msg_as_string);
     if ($Conf{'key_passwd'} ne '') {
 	# if password is define in sympa.conf pass the password to OpenSSL using
-	$pass_option = "-passin file:$Conf{'tmpdir'}/pass.$$";	
+	$pass_option = "-passin file:$temporary_pwd";	
     }
 
     ## try all keys/certs until one decrypts.
@@ -766,8 +831,8 @@ sub smime_decrypt {
 	my $keyfile = shift @$keys;
 	&do_log('debug', "Trying decrypt with $certfile, $keyfile");
 	if ($Conf{'key_passwd'} ne '') {
-	    unless (&POSIX::mkfifo("$Conf{'tmpdir'}/pass.$$",0600)) {
-		&do_log('err', 'Unable to make fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
+	    unless (&POSIX::mkfifo($temporary_pwd,0600)) {
+		&do_log('err', 'Unable to make fifo for %s', $temporary_pwd);
 		return undef;
 	    }
 	}
@@ -776,13 +841,13 @@ sub smime_decrypt {
 	open (NEWMSG, "$Conf{'openssl'} smime -decrypt -in $temporary_file -recip $certfile -inkey $keyfile $pass_option |");
 
 	if ($Conf{'key_passwd'} ne '') {
-	    unless (open (FIFO,"> $Conf{'tmpdir'}/pass.$$")) {
-		&do_log('notice', 'Unable to open fifo for %s/pass.%s',$Conf{'tmpdir'},$$);
+	    unless (open (FIFO,"> $temporary_pwd")) {
+		&do_log('notice', 'Unable to open fifo for %s', $temporary_pwd);
 		return undef;
 	    }
 	    print FIFO $Conf{'key_passwd'};
 	    close FIFO;
-	    unlink ("$Conf{'tmpdir'}/pass.$$");
+	    unlink ($temporary_pwd);
 	}
 	
 	while (<NEWMSG>) {
@@ -836,10 +901,18 @@ sub smime_decrypt {
 ## Make a multipart/alternative, a singlepart
 sub as_singlepart {
     &do_log('debug2', 'tools::as_singlepart()');
-    my ($msg, $preferred_type) = @_;
+    my ($msg, $preferred_type, $loops) = @_;
     my $done = 0;
+    $loops++;
     
-    if ($msg->effective_type() =~ /^multipart\/alternative/) {
+    if ($loops > 4) {
+	do_log('err', 'Could not change multipart to singlepart');
+	return undef;
+    }
+
+    if ($msg->effective_type() =~ /^$preferred_type$/) {
+	$done = 1;
+    }elsif ($msg->effective_type() =~ /^multipart\/alternative/) {
 	my @parts = $msg->parts();
 	foreach my $index (0..$#parts) {
 	    if (($parts[$index]->effective_type() =~ /^$preferred_type$/) ||
@@ -853,11 +926,23 @@ sub as_singlepart {
 		last;
 	    }
 	}
+    }elsif ($msg->effective_type() =~ /multipart\/signed/) {
+	my @parts = $msg->parts();
+	## Only keep the first part
+	$msg->parts([$parts[0]]);
+	$msg->make_singlepart();
+	
+	$done ||= &as_singlepart($msg, $preferred_type, $loops);
+
     }elsif ($msg->effective_type() =~ /^multipart/) {
 	my @parts = $msg->parts();
 	foreach my $index (0..$#parts) {
 	    if ($parts[$index]->effective_type() =~ /^multipart\/alternative/) {
-		$done ||= &as_singlepart($parts[$index], $preferred_type);
+		if (&as_singlepart($parts[$index], $preferred_type, $loops)) {
+		    $msg->parts([$parts[$index]]);
+		    $msg->make_singlepart();
+		    $done = 1;
+		}
 	    }
 	}    
     }
@@ -1124,7 +1209,7 @@ sub virus_infected {
     &do_log('debug2', 'Scan virus in %s', $file);
     
     unless ($Conf{'antivirus_path'} ) {
-        &do_log('debug2', 'Sympa not configured to scan virus in message');
+        &do_log('debug', 'Sympa not configured to scan virus in message');
 	return 0;
     }
     my @name = split(/\//,$file);
@@ -1132,14 +1217,14 @@ sub virus_infected {
     
     unless ((-d $work_dir) ||( mkdir $work_dir, 0755)) {
 	do_log('err', "Unable to create tmp antivirus directory $work_dir");
-	return 0;
+	return undef;
     }
 
     $work_dir = $Conf{'tmpdir'}.'/antivirus/'.$name[$#name];
     
-    unless ( mkdir ($work_dir, 0755)) {
+    unless ( (-d $work_dir) || mkdir ($work_dir, 0755)) {
 	do_log('err', "Unable to create tmp antivirus directory $work_dir");
-	return 0;
+	return undef;
     }
 
     #$mail->dump_skeleton;
@@ -1150,7 +1235,7 @@ sub virus_infected {
 	return undef;
     }
 
-    my $virusfound; 
+    my $virusfound = 0; 
     my $error_msg;
     my $result;
 
@@ -1158,7 +1243,10 @@ sub virus_infected {
     if ($Conf{'antivirus_path'} =~  /\/uvscan$/) {
 
 	# impossible to look for viruses with no option set
-	return 0 unless ($Conf{'antivirus_args'});
+	unless ($Conf{'antivirus_args'}) {
+	    &do_log('err', "Missing 'antivirus_args' in sympa.conf");
+	    return undef;
+	}
     
 	open (ANTIVIR,"$Conf{'antivirus_path'} $Conf{'antivirus_args'} $work_dir |") ; 
 		
@@ -1173,13 +1261,19 @@ sub virus_infected {
     
 	my $status = $?/256 ;
 
-        ## uvscan status =13 (*256) => virus
-        if (( $status == 13) and not($virusfound)) { 
-	    $virusfound = "unknown";
+        ## uvscan status =12 or 13 (*256) => virus
+        if (( $status == 13) || ($status == 12)) { 
+	    $virusfound ||= "unknown";
 	}
 
+	## Meaning of the codes
+	##  12 : The program tried to clean a file, and that clean failed for some reason and the file is still infected.
+	##  13 : One or more viruses or hostile objects (such as a Trojan horse, joke program,  or  a  test file) were found.
+	##  15 : The programs self-check failed; the program might be infected or damaged.
+	##  19 : The program succeeded in cleaning all infected files.
+
 	$error_msg = $result
-	    if ($status != 0 && $status != 13);
+	    if ($status != 0 && $status != 12 && $status != 13 && $status != 19);
 
     ## Trend Micro
     }elsif ($Conf{'antivirus_path'} =~  /\/vscan$/) {
@@ -1205,7 +1299,10 @@ sub virus_infected {
 	$dbdir=$` ;
 
 	# impossible to look for viruses with no option set
-	return 0 unless ($Conf{'antivirus_args'});
+	unless ($Conf{'antivirus_args'}) {
+	    &do_log('err', "Missing 'antivirus_args' in sympa.conf");
+	    return undef;
+	}
 
 	open (ANTIVIR,"$Conf{'antivirus_path'} --databasedirectory $dbdir $Conf{'antivirus_args'} $work_dir |") ;
 
@@ -1278,7 +1375,10 @@ sub virus_infected {
     }elsif ("${Conf{'antivirus_path'}}" =~ /\/sweep$/) {
 	
         # impossible to look for viruses with no option set
-        return 0 unless ($Conf{'antivirus_args'});
+	unless ($Conf{'antivirus_args'}) {
+	    &do_log('err', "Missing 'antivirus_args' in sympa.conf");
+	    return undef;
+	}
     
         open (ANTIVIR,"$Conf{'antivirus_path'} $Conf{'antivirus_args'} $work_dir |") ;
 	
@@ -1297,11 +1397,13 @@ sub virus_infected {
 	}
 
 	## Clam antivirus
-    }elsif ("${Conf{'antivirus_path'}}" =~ /\/clamscan$/) {
+    }elsif ("${Conf{'antivirus_path'}}" =~ /\/clamd?scan$/) {
 	
         open (ANTIVIR,"$Conf{'antivirus_path'} $Conf{'antivirus_args'} $work_dir |") ;
 	
+	my $result;
 	while (<ANTIVIR>) {
+	    $result .= $_; chomp $result;
 	    if (/^\S+:\s(.*)\sFOUND$/) {
 		$virusfound = $1;
 	    }
@@ -1314,11 +1416,20 @@ sub virus_infected {
 	if (( $status == 1) and not($virusfound)) {
 	    $virusfound = "unknown";
 	}
+
+	$error_msg = $result
+	    if ($status != 0 && $status != 1);
+
     }         
 
     ## Error while running antivir, notify listmaster
     if ($error_msg) {
-	&List::send_notify_to_listmaster('virus_scan_failed', $Conf{'domain'}, ($file,$error_msg));
+	unless (&List::send_notify_to_listmaster('virus_scan_failed', $Conf{'domain'},
+						 {'filename' => $file,
+						  'error_msg' => $error_msg})) {
+	    &do_log('notice',"Unable to send notify 'virus_scan_failed' to listmaster");
+	}
+
     }
 
     ## if debug mode is active, the working directory is kept
@@ -1332,7 +1443,7 @@ sub virus_infected {
 	rmdir ($work_dir) ;
     }
    
-    return ($virusfound);
+    return $virusfound;
    
 }
 
@@ -1460,16 +1571,31 @@ sub duration_conv {
 
 ## Look for a file in the list > robot > server > default locations
 sub get_filename {
-    my ($type, $name, $robot, $list) = @_;
-    &do_log('debug3','tools::get_filename(%s,%s,%s,%s)', $type, $name, $robot, $list->{'name'});
-
+    my ($type, $name, $robot, $object) = @_;
+    my $list;
+    my $family;
+    &do_log('debug3','tools::get_filename(%s,%s,%s,%s)', $type, $name, $robot, $object->{'name'});
+    
+    if (ref($object) eq 'List') {
+ 	$list = $object;
+ 	if ($list->{'admin'}{'family_name'}) {
+ 	    unless ($family = $list->get_family()) {
+ 		&do_log('err', 'Impossible to get list %s family : %s. The list is set in status error_config',$self->{'name'},$self->{'admin'}{'family_name'});
+ 		$list->set_status_error_config('no_list_family',$self->{'name'}, $admin->{'family_name'});
+ 		return undef;
+ 	    }  
+ 	}
+    }elsif (ref($object) eq 'Family') {
+ 	$family = $object;
+    }
+    
     if ($type eq 'etc') {
 	my (@try, $default_name);
 	
 	## template refers to a language
 	## => extend search to default tpls
-	if ($name =~ /^(\S+)\.(\S+)\.tpl$/) {
-	    $default_name = $1.'.tpl';
+	if ($name =~ /^(\S+)\.([^\s\/]+)\.tt2$/) {
+	    $default_name = $1.'.tt2';
 	    
 	    @try = ("$Conf{'etc'}/$robot".'/'.$name,
 		    "$Conf{'etc'}/$robot".'/'.$default_name,
@@ -1482,25 +1608,24 @@ sub get_filename {
 		    $Conf{'etc'}.'/'.$name,
 		    '--ETCBINDIR--'.'/'.$name);
 	}
-	if ($list) {
+	
+	if ($family) {
+ 	    ## Default tpl
+ 	    if ($default_name) {
+		unshift @try, $family->{'dir'}.'/'.$default_name;
+	    }
+	}
+	
+	unshift @try, $family->{'dir'}.'/'.$name;
+    
+	if ($list->{'name'}) {
 	    ## Default tpl
 	    if ($default_name) {
-		## No 'templates' subdir in list directory
-		if ($default_name =~ /^templates\/(.*)$/) {
-		    unshift @try, $list->{'dir'}.'/'.$1;
-		}else {
-		    unshift @try, $list->{'dir'}.'/'.$default_name;
-		}
+		unshift @try, $list->{'dir'}.'/'.$default_name;
 	    }
-
-	    ## Also look for templates in the list's directory
-	    if ($name =~ /^templates\/(.*)$/) {
-		unshift @try, $list->{'dir'}.'/'.$name;
-		unshift @try, $list->{'dir'}.'/'.$1;
-	    }else {
-		unshift @try, $list->{'dir'}.'/'.$name;
-	    }
-	}	
+	    
+	    unshift @try, $list->{'dir'}.'/'.$name;
+	}
 	foreach my $f (@try) {
 	    &do_log('debug3','get_filname : NAME: %s ; DIR %s', $name, $f  );
 	    if (-r $f) {
@@ -1509,7 +1634,107 @@ sub get_filename {
 	}
     }
     
-    &do_log('debug3','tools::get_filename: Cannot find %s', $name);
+    &do_log('notice','tools::get_filename: Cannot find %s in %s', $name, join(',',@try));
+    return undef;
+}
+####################################################
+# make_tt2_include_path
+####################################################
+# make an array of include path for tt2 parsing
+# 
+# IN -$robot(+) : robot
+#    -$dir : directory ending each path
+#    -$lang : lang
+#    -$list : ref(List)
+#
+# OUT : ref(ARRAY) of tt2 include path
+#
+######################################################
+sub make_tt2_include_path {
+    my ($robot,$dir,$lang,$list) = @_;
+    &do_log('debug3','tools::make_tt2_include_path(%s,%s,%s,%s)',$robot,$dir,$lang,$list);
+
+    my @include_path;
+
+    my $path_etcbin;
+    my $path_etc;
+    my $path_robot;  ## optional
+    my $path_list;   ## optional
+    my $path_family; ## optional
+
+    if ($dir) {
+	$path_etcbindir = "--ETCBINDIR--/".$dir;
+	$path_etcdir = "$Conf{'etc'}/".$dir;
+	$path_robot = "$Conf{'etc'}/".$robot.'/'.$dir if (lc($robot) ne lc($Conf{'host'}));
+	if (ref($list) eq 'List'){
+	    $path_list = $list->{'dir'}.'/'.$dir;
+	    if (defined $list->{'admin'}{'family_name'}) {
+		my $family = $list->get_family();
+	        $path_family = $family->{'dir'}.'/'.$dir;
+	    }
+	} 
+    }else {
+	$path_etcbindir = "--ETCBINDIR--";
+	$path_etcdir = "$Conf{'etc'}";
+	$path_robot = "$Conf{'etc'}/".$robot if (lc($robot) ne lc($Conf{'host'}));
+	if (ref($list) eq 'List') {
+	    $path_list = $list->{'dir'} ;
+	    if (defined $list->{'admin'}{'family_name'}) {
+		my $family = $list->get_family();
+	        $path_family = $family->{'dir'};
+	    }
+	}
+    }
+    if ($lang) {
+	@include_path = ($path_etcdir.'/'.$lang,
+			 $path_etcdir,
+			 $path_etcbindir.'/'.$lang,
+			 $path_etcbindir);
+	if ($path_robot) {
+	    unshift @include_path,$path_robot;
+	    unshift @include_path,$path_robot.'/'.$lang;
+	}
+	if ($path_list) {
+	    unshift @include_path,$path_list;
+	    unshift @include_path,$path_list.'/'.$lang;
+
+	    if ($path_family) {
+		unshift @include_path,$path_family;
+		unshift @include_path,$path_family.'/'.$lang;
+	    }	
+	    
+	}
+    }else {
+	@include_path = ($path_etcdir,
+			 $path_etcbindir);
+
+	if ($path_robot) {
+	    unshift @include_path,$path_robot;
+	}
+	if ($path_list) {
+	    unshift @include_path,$path_list;
+	   
+	    if ($path_family) {
+		unshift @include_path,$path_family;
+	    }
+	}
+    }
+
+    return \@include_path;
+
+}
+
+## Find a file in an ordered list of directories
+sub find_file {
+    my ($filename, @directories) = @_;
+    &do_log('debug3','tools::find_file(%s,%s)', $filename, join(':',@directories));
+
+    foreach my $d (@directories) {
+	if (-f "$d/$filename") {
+	    return "$d/$filename";
+	}
+    }
+    
     return undef;
 }
 
@@ -1531,16 +1756,16 @@ sub write_pid {
 
     ## Create and write the pidfile
     unless (open(LOCK, "+>> $pidfile")) {
-	 fatal_err("Could not open %s, exiting", $pidfile);
+	 fatal_err('Could not open %s, exiting', $pidfile);
     } 
     unless (flock(LOCK, 6)) {
-	fatal_err("Could not lock %s, process is probably already running : %s", $pidfile, $!);
+	fatal_err('Could not lock %s, process is probably already running : %s', $pidfile, $!);
     }
     unless (open(LCK, "> $pidfile")) {
-	fatal_err("Could not open %s, exiting", $pidfile);
+	fatal_err('Could not open %s, exiting', $pidfile);
     }
     unless (truncate(LCK, 0)) {
-	fatal_err("Could not truncate %s, exiting.", $pidfile);
+	fatal_err('Could not truncate %s, exiting.', $pidfile);
     }
     
     print LCK "$pid\n";
@@ -1585,7 +1810,18 @@ sub get_dir_size {
 sub valid_email {
     my $email = shift;
     
-    $email =~ /^$tools::regexp{'email'}$/;
+    unless ($email =~ /^$tools::regexp{'email'}$/) {
+	do_log('err', "Invalid email address '%s'", $email);
+	return undef;
+    }
+    
+    ## Forbidden characters
+    if ($email =~ /[\|\$\*\?\!]/) {
+	do_log('err', "Invalid email address '%s'", $email);
+	return undef;
+    }
+
+    return 1;
 }
 
 ## Clean email address
@@ -1602,39 +1838,6 @@ sub clean_email {
     return $email;
 }
 
-## Function for Removing a non-empty directory
-## It takes a variale number of arguments : 
-## it can be a list of directory
-## or few direcoty paths
-sub remove_dir {
-    
-    do_log('info','remove_dir()');
-    
-    foreach my $current_dir (@_){
-	my @tree = split /\//, $current_dir ;
-	if ($#tree < 4) {
-	    do_log('info',"$current_dir not removed (not enough / in directory name)");
-	    next;
-	}
-	finddepth({wanted => \&del, no_chdir => 1},$current_dir);
-    }
-    sub del {
-	my $name = $File::Find::name;
-
-	if (!-l && -d _) {
-	    &do_log('info','removing dir %s',$name);
-	    unless (rmdir($name)) {
-		&do_log('info','Error while removing dir %s',$name);
-	    }
-	}else{
-	    unless (unlink($name)) {
-		&do_log('info','Error while removing file  %s',$name);
-	    }
-	}
-    }
-    return 1;
-}
-
 ## Return canonical email address (lower-cased + space cleanup)
 ## It could also support alternate email
 sub get_canonical_email {
@@ -1649,6 +1852,38 @@ sub get_canonical_email {
     return $email;
 }
 
+## Function for Removing a non-empty directory
+## It takes a variale number of arguments : 
+## it can be a list of directory
+## or few direcoty paths
+sub remove_dir {
+    
+    do_log('debug2','remove_dir()');
+    
+    foreach my $current_dir (@_){
+	my @tree = split /\//, $current_dir ;
+	if ($#tree < 4) {
+	    do_log('err',"$current_dir not removed (not enough / in directory name)");
+	    next;
+	}
+	finddepth({wanted => \&del, no_chdir => 1},$current_dir);
+    }
+    sub del {
+	my $name = $File::Find::name;
+
+	if (!-l && -d _) {
+	    unless (rmdir($name)) {
+		&do_log('err','Error while removing dir %s',$name);
+	    }
+	}else{
+	    unless (unlink($name)) {
+		&do_log('err','Error while removing file  %s',$name);
+	    }
+	}
+    }
+    return 1;
+}
+
 ## find the appropriate S/MIME keys/certs for $oper in $dir.
 ## $oper can be:
 ## 'sign' -> return the preferred signing key/cert
@@ -1835,6 +2070,8 @@ sub dump_var {
 		print $fd "\t"x$level.'_'.$key.'_'."\n";
 		&dump_var($var->{$key}, $level+1, $fd);
 	    }    
+	}else {
+	    printf $fd "\t"x$level."'%s'"."\n", ref($var);
 	}
     }else {
 	if (defined $var) {
@@ -1845,6 +2082,181 @@ sub dump_var {
     }
 }
 
+sub remove_empty_entries {
+    my ($var) = @_;    
+    my $not_empty = 0;
+
+    if (ref($var)) {
+	if (ref($var) eq 'ARRAY') {
+	    foreach my $index (0..$#{$var}) {
+		my $status = &remove_empty_entries($var->[$index]);
+		$var->[$index] = undef unless ($status);
+		$not_empty ||= $status
+	    }	    
+	}elsif (ref($var) eq 'HASH') {
+	    foreach my $key (sort keys %{$var}) {
+		my $status = &remove_empty_entries($var->{$key});
+		$var->{$key} = undef unless ($status);
+		$not_empty ||= $status;
+	    }    
+	}
+    }else {
+	if (defined $var && $var) {
+	    $not_empty = 1
+	}
+    }
+    
+    return $not_empty;
+}
+
+## Duplictate a complex variable
+sub dup_var {
+    my ($var) = @_;    
+
+    if (ref($var)) {
+	if (ref($var) eq 'ARRAY') {
+	    my $new_var = [];
+	    foreach my $index (0..$#{$var}) {
+		$new_var->[$index] = &dup_var($var->[$index]);
+	    }	    
+	    return $new_var;
+	}elsif (ref($var) eq 'HASH') {
+	    my $new_var = {};
+	    foreach my $key (sort keys %{$var}) {
+		$new_var->{$key} = &dup_var($var->{$key});
+	    }    
+	    return $new_var;
+	}
+    }
+    
+    return $var; 
+}
+
+####################################################
+# get_array_from_splitted_string                          
+####################################################
+# return an array made on a string splited by ','.
+# It removes spaces.
+#
+# 
+# IN : -$string (+): string to split 
+#
+# OUT : -ref(ARRAY)
+#
+######################################################
+sub get_array_from_splitted_string {
+    my ($string) = @_;
+    my @array;
+
+    foreach my $word (split /,/,$string) {
+	$word =~ s/^\s+//;
+	$word =~ s/\s+$//;
+	push @array, $word;
+    }
+
+    return \@array;
+}
+
+
+####################################################
+# diff_on_arrays                     
+####################################################
+# Makes set operation on arrays (seen as set, with no double) :
+#  - deleted : A \ B
+#  - added : B \ A
+#  - intersection : A /\ B
+#  - union : A \/ B
+# 
+# IN : -$setA : ref(ARRAY) - set
+#      -$setB : ref(ARRAY) - set
+#
+# OUT : -ref(HASH) with keys :  
+#          deleted, added, intersection, union
+#
+#######################################################    
+sub diff_on_arrays {
+    my ($setA,$setB) = @_;
+    my $result = {'intersection' => [],
+	          'union' => [],
+	          'added' => [],
+	          'deleted' => []};
+    my %deleted;
+    my %added;
+    my %intersection;
+    my %union;
+    
+    my %hashA;
+    my %hashB;
+    
+    foreach my $eltA (@$setA) {
+	$hashA{$eltA} = 1;
+	$deleted{$eltA} = 1;
+	$union{$eltA} = 1;
+    }
+    
+    foreach my $eltB (@$setB) {
+	$hashB{$eltB} = 1;
+	$added{$eltB} = 1;
+	
+	if ($hashA{$eltB}) {
+	    $intersection{$eltB} = 1;
+	    $deleted{$eltB} = 0;
+	}else {
+	    $union{$eltB} = 1;
+	}
+    }
+    
+    foreach my $eltA (@$setA) {
+	if ($hashB{$eltA}) {
+	    $added{$eltA} = 0; 
+	}
+    }
+    
+    foreach my $elt (keys %deleted) {
+	next unless $elt;
+	push @{$result->{'deleted'}},$elt if ($deleted{$elt});
+    }
+    foreach my $elt (keys %added) {
+	next unless $elt;
+	push @{$result->{'added'}},$elt if ($added{$elt});
+    }
+    foreach my $elt (keys %intersection) {
+	next unless $elt;
+	push @{$result->{'intersection'}},$elt if ($intersection{$elt});
+    }
+    foreach my $elt (keys %union) {
+	next unless $elt;
+	push @{$result->{'union'}},$elt if ($union{$elt});
+    } 
+    
+    return $result;
+    
+} 
+
+####################################################
+# clean_msg_id
+####################################################
+# clean msg_id to use it without  \n, \s or <,>
+# 
+# IN : -$msg_id (+) : the msg_id
+#
+# OUT : -$msg_id : the clean msg_id
+#
+######################################################
+sub clean_msg_id {
+    my $msg_id = shift;
+    
+    chomp $msg_id;
+
+    if ($msg_id =~ /\<(.+)\>/) {
+	$msg_id = $1;
+    }
+
+    return $msg_id;
+}
+
+
+
 ## Change X-Sympa-To: header field in the message
 sub change_x_sympa_to {
     my ($file, $value) = @_;
@@ -1872,6 +2284,167 @@ sub change_x_sympa_to {
     return 1;
 }
 
+## Compare 2 versions of Sympa
+sub higher_version {
+    my ($v1, $v2) = @_;
+
+    my @tab1 = split /\./,$v1;
+    my @tab2 = split /\./,$v2;
+    
+    
+    my $max = $#tab1;
+    $max = $#tab2 if ($#tab2 > $#tab1);
+
+    for $i (0..$max) {
+    
+        if ($tab1[0] =~ /^(\d*)a$/) {
+            $tab1[0] = $1 - 0.5;
+        }elsif ($tab1[0] =~ /^(\d*)b$/) {
+            $tab1[0] = $1 - 0.25;
+        }
+
+        if ($tab2[0] =~ /^(\d*)a$/) {
+            $tab2[0] = $1 - 0.5;
+        }elsif ($tab2[0] =~ /^(\d*)b$/) {
+            $tab2[0] = $1 - 0.25;
+        }
+
+        if ($tab1[0] eq $tab2[0]) {
+            #printf "\t%s = %s\n",$tab1[0],$tab2[0];
+            shift @tab1;
+            shift @tab2;
+            next;
+        }
+        return ($tab1[0] > $tab2[0]);
+    }
+
+    return 0;
+}
+
+## Compare 2 versions of Sympa
+sub lower_version {
+    my ($v1, $v2) = @_;
+
+    my @tab1 = split /\./,$v1;
+    my @tab2 = split /\./,$v2;
+    
+    
+    my $max = $#tab1;
+    $max = $#tab2 if ($#tab2 > $#tab1);
+
+    for $i (0..$max) {
+    
+        if ($tab1[0] =~ /^(\d*)a$/) {
+            $tab1[0] = $1 - 0.5;
+        }elsif ($tab1[0] =~ /^(\d*)b$/) {
+            $tab1[0] = $1 - 0.25;
+        }
+
+        if ($tab2[0] =~ /^(\d*)a$/) {
+            $tab2[0] = $1 - 0.5;
+        }elsif ($tab2[0] =~ /^(\d*)b$/) {
+            $tab2[0] = $1 - 0.25;
+        }
+
+        if ($tab1[0] eq $tab2[0]) {
+            #printf "\t%s = %s\n",$tab1[0],$tab2[0];
+            shift @tab1;
+            shift @tab2;
+            next;
+        }
+        return ($tab1[0] < $tab2[0]);
+    }
+
+    return 0;
+}
+
+sub LOCK_SH {1};
+sub LOCK_EX {2};
+sub LOCK_NB {4};
+sub LOCK_UN {8};
+
+
+
+## lock a file 
+sub lock {
+    my $lock_file = shift;
+    my $mode = shift; ## read or write
+    
+    my $operation;
+    my $open_mode;
+
+    if ($mode eq 'read') {
+	$operation = LOCK_SH;
+    }else {
+	$operation = LOCK_EX;
+	$open_mode = '>>';
+    }
+    
+    ## Read access to prevent "Bad file number" error on Solaris
+    unless (open FH, $open_mode.$lock_file) {
+	&do_log('err', 'Cannot open %s: %s', $lock_file, $!);
+	return undef;
+    }
+    
+    my $got_lock = 1;
+    unless (flock (FH, $operation | LOCK_NB)) {
+	&do_log('notice','Waiting for %s lock on %s', $mode, $lock_file);
+
+	## If lock was obtained more than 20 minutes ago, then force the lock
+	if ( (time - (stat($lock_file))[9] ) >= 60*20) {
+	    &do_log('notice','Removing lock file %s', $lock_file);
+	    unless (unlink $lock_file) {
+		&do_log('err', 'Cannot remove %s: %s', $lock_file, $!);
+		return undef;	    		
+	    }
+	    
+	    unless (open FH, ">$lock_file") {
+		&do_log('err', 'Cannot open %s: %s', $lock_file, $!);
+		return undef;	    
+	    }
+	}
+
+	$got_lock = undef;
+	my $max = 10;
+	$max = 2 if ($ENV{'HTTP_HOST'}); ## Web context
+	for (my $i = 1; $i < $max; $i++) {
+	    sleep (10 * $i);
+	    if (flock (FH, $operation | LOCK_NB)) {
+		$got_lock = 1;
+		last;
+	    }
+	    &do_log('notice','Waiting for %s lock on %s', $mode, $lock_file);
+	}
+    }
+	
+    if ($got_lock) {
+	&do_log('debug2', 'Got lock for %s on %s', $mode, $lock_file);
+
+	## Keep track of the locking PID
+	if ($mode eq 'write') {
+	    print FH "$$\n";
+	}
+    }else {
+	&do_log('err', 'Failed locking %s: %s', $lock_file, $!);
+	return undef;
+    }
+
+    return \*FH;
+}
+
+## unlock a file 
+sub unlock {
+    my $lock_file = shift;
+    my $fh = shift;
+    
+    unless (flock($fh,LOCK_UN)) {
+	&do_log('err', 'Failed UNlocking %s: %s', $lock_file, $!);
+	return undef;
+    }
+    close $fh;
+    &do_log('debug2', 'Release lock on %s', $lock_file);
+    
+    return 1;
+}
 
 1;
-
