@@ -1,5 +1,5 @@
 # wwslib.pm - This module includes functions used by wwsympa.fcgi
-# RCS Identication ; $Revision: 1.10.2.1 $ ; $Date: 2004/08/05 09:18:12 $ 
+# RCS Identication ; $Revision: 1.20 $ ; $Date: 2006/02/28 16:44:59 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -30,6 +30,7 @@ use Exporter;
 use Log;
 use Conf;
 use List;
+use report;
 
 # use Net::SSLeay qw(&get_https);
 # use Net::SSLeay;
@@ -38,6 +39,7 @@ use List;
 
  ## authentication : via email or uid
  sub check_auth{
+     my $robot = shift;
      my $auth = shift; ## User email or UID
      my $pwd = shift; ## Password
      &do_log('debug', 'Auth::check_auth(%s)', $auth);
@@ -45,11 +47,11 @@ use List;
      my ($canonic, $user);
 
      if( &tools::valid_email($auth)) {
-	 return &authentication($auth,$pwd);
+	 return &authentication($robot, $auth,$pwd);
 
      }else{
 	 ## This is an UID
-	 if ($canonic = &ldap_authentication($auth,$pwd,'uid_filter')){
+	 if ($canonic = &ldap_authentication($robot, $auth,$pwd,'uid_filter')){
 
 	     unless($user = &List::get_user_db($canonic)){
 		 $user = {'email' => $canonic};
@@ -60,7 +62,7 @@ use List;
 		 };
 	     
 	 }else{
-	     &main::error_message('incorrect_passwd') unless ($ENV{'SYMPA_SOAP'});
+	     &report::reject_report_web('user','incorrect_passwd',{}) unless ($ENV{'SYMPA_SOAP'});
 	     &do_log('err', "Incorrect Ldap password");
 	     return undef;
 	 }
@@ -69,9 +71,9 @@ use List;
 
 
 sub authentication {
-    my ($email,$pwd) = @_;
+    my ($robot, $email,$pwd) = @_;
     my ($user,$canonic);
-    &do_log('debug', 'Auth::authentication(%s)', $auth);
+    &do_log('debug', 'Auth::authentication(%s)', $email);
 
 
     unless ($user = &List::get_user_db($email)) {
@@ -85,7 +87,7 @@ sub authentication {
     
 
 
-    foreach my $auth_service (@{$Conf{'auth_services'}}){
+    foreach my $auth_service (@{$Conf{'auth_services'}{$robot}}){
 	next if ($email !~ /$auth_service->{'regexp'}/i);
 	next if (($email =~ /$auth_service->{'negative_regexp'}/i)&&($auth_service->{'negative_regexp'}));
 	if ($auth_service->{'auth_type'} eq 'user_table') {
@@ -98,7 +100,7 @@ sub authentication {
 			};
 	    }
 	}elsif($auth_service->{'auth_type'} eq 'ldap') {
-	    if ($canonic = &ldap_authentication($email,$pwd,'email_filter')){
+	    if ($canonic = &ldap_authentication($robot, $email,$pwd,'email_filter')){
 		unless($user = &List::get_user_db($canonic)){
 		    $user = {'email' => $canonic};
 		}
@@ -113,19 +115,19 @@ sub authentication {
     ## If web context and password has never been changed
     ## Then prompt user
     unless ($ENV{'SYMPA_SOAP'}) {
-	foreach my $auth_service (@{$Conf{'auth_services'}}){
+	foreach my $auth_service (@{$Conf{'auth_services'}{$robot}}){
 	    next unless ($email !~ /$auth_service->{'regexp'}/i);
 	    next unless (($email =~ /$auth_service->{'negative_regexp'}/i)&&($auth_service->{'negative_regexp'}));
 	    if ($auth_service->{'auth_type'} eq 'user_table') {
 		if ($user->{'password'} =~ /^init/i) {
-		    &main::error_message('init_passwd');
+		    &report::reject_report_web('user','init_passwd',{});
 		    last;
 		}
 	    }
 	}
     }
     
-    &main::error_message('incorrect_passwd') unless ($ENV{'SYMPA_SOAP'});
+    &report::reject_report_web('user','incorrect_passwd',{}) unless ($ENV{'SYMPA_SOAP'});
     &do_log('err','authentication: incorrect password for user %s', $email);
 
     $param->{'init_email'} = $email;
@@ -135,7 +137,7 @@ sub authentication {
 
 
 sub ldap_authentication {
-     my ($auth,$pwd,$whichfilter) = @_;
+     my ($robot, $auth,$pwd,$whichfilter) = @_;
      my ($cnx, $mesg, $host,$ldap_passwd,$ldap_anonymous);
      &do_log('debug2','Auth::ldap_authentication(%s,%s,%s)', $auth,$pwd,$whichfilter);
 
@@ -144,7 +146,7 @@ sub ldap_authentication {
      }
 
      ## No LDAP entry is defined in auth.conf
-     if ($#{$Conf{'auth_services'}} < 0) {
+     if ($#{$Conf{'auth_services'}{$robot}} < 0) {
 	 &do_log('notice', 'Skipping empty auth.conf');
 	 return undef;
      }
@@ -167,7 +169,7 @@ sub ldap_authentication {
      }
      require Net::LDAP::Message;
      
-     foreach my $ldap (@{$Conf{'auth_services'}}){
+     foreach my $ldap (@{$Conf{'auth_services'}{$robot}}){
 	 # only ldap service are to be applied here
 	 next unless ($ldap->{'auth_type'} eq 'ldap');
 
@@ -278,7 +280,7 @@ sub ldap_authentication {
 					   );
 
 	     if ($mesg->count() == 0) {
-		 do_log('notice',"No entry in the Ldap Directory Tree of %s,$host");
+		 do_log('notice',"No entry in the Ldap Directory Tree of %s", $host);
 		 $ldap_passwd->unbind;
 		 last;
 	     }
@@ -329,13 +331,25 @@ sub ldap_authentication {
 
 
 # fetch user email using his cas net_id and the paragrapah number in auth.conf
-sub cas_get_email_by_net_id {
+sub get_email_by_net_id {
     
-    my $net_id = shift;   
+    my $robot = shift;
     my $auth_id = shift;
-
-    do_log ('info',"Auth::cas_get_email_by_net_id($net_id,$auth_id)");
-
+    my $attributes = shift;
+    
+    do_log ('debug',"Auth::get_email_by_net_id($auth_id,$attributes->{'uid'})");
+    
+    if (defined $Conf{'auth_services'}{$robot}[$auth_id]{'internal_email_by_netid'}) {
+	my $sso_config = @{$Conf{'auth_services'}{$robot}}[$auth_id];
+	my $netid_cookie = $sso_config->{'netid_http_header'} ;
+	
+	$netid_cookie =~ s/(\w+)/$attributes->{$1}/ig;
+	
+	$email = &List::get_netidtoemail_db($robot, $netid_cookie, $Conf{'auth_services'}{$robot}[$auth_id]{'service_id'});
+	
+	return $email;
+    }
+ 
     unless (eval "require Net::LDAP") {
 	do_log ('err',"Unable to use LDAP library, Net::LDAP required, install perl-ldap (CPAN) first");
 	return undef;
@@ -354,10 +368,10 @@ sub cas_get_email_by_net_id {
     }
     require Net::LDAP::Message;
 
-    my $ldap = @{$Conf{'auth_services'}}[$auth_id];
+    my $ldap = @{$Conf{'auth_services'}{$robot}}[$auth_id];
     my $filter = $ldap->{'ldap_get_email_by_uid_filter'} ;
-    $filter =~ s/\[uid\]/$net_id/ig;
 
+    $filter =~ s/\[([\w-]+)\]/$attributes->{$1}/ig;
 
     foreach my $host (split(/,/,$ldap->{'ldap_host'})){
 
@@ -391,7 +405,7 @@ sub cas_get_email_by_net_id {
 	
 	my $cnx;
 	## Not always anonymous...
-	if (defined ($ldap->{'bind_dn'}) && defined ($ldap->{'bind_password'})) {
+	if (defined ($ldap->{'ldap_bind_dn'}) && defined ($ldap->{'ldap_bind_password'})) {
 	    $cnx = $ldap_anonymous->bind($ldap->{'ldap_bind_dn'}, password =>$ldap->{'ldap_bind_password'});
 	}else {
 	    $cnx = $ldap_anonymous->bind;
@@ -414,7 +428,7 @@ sub cas_get_email_by_net_id {
 	my $count = $emails->count();
 
 	if ($emails->count() == 0) {
-	    do_log('notice',"No entry in the Ldap Directory Tree of %s,$host");
+	    do_log('notice',"No entry in the Ldap Directory Tree of %s", $host);
 	    $ldap_anonymous->unbind;
 	    last;
 	}
