@@ -1,5 +1,5 @@
 # wwslib.pm - This module includes functions used by wwsympa.fcgi
-# RCS Identication ; $Revision: 1.43 $ ; $Date: 2004/03/05 09:25:24 $ 
+# RCS Identication ; $Revision: 1.53 $ ; $Date: 2006/01/27 14:54:23 $ 
 #
 # Sympa - SYsteme de Multi-Postage Automatique
 # Copyright (c) 1997, 1998, 1999, 2000, 2001 Comite Reseau des Universites
@@ -32,50 +32,50 @@ use Conf;
 # use Net::SSLeay qw(&get_https);
 # use Net::SSLeay;
 
-## Supported web languages
-@languages = ('fr','us','es','it','nl','cn','cz','de','hu','et','ro','fi');
-
-%reception_mode = ('mail' => 'normal',
-		   'digest' => 'digest',
-		   'summary' => 'summary',
-		   'notice' => 'notice',
-		   'txt' => 'txt',
-		   'html'=> 'html',
-		   'urlize' => 'urlize',
-		   'nomail' => 'no mail',
-		   'not_me' => 'not_me');
+%reception_mode = ('mail' => {'gettext_id' => 'standard (direct reception)'},
+		   'digest' => {'gettext_id' => 'digest MIME format'},
+		   'digestplain' => {'gettext_id' => 'digest plain text format'},
+		   'summary' => {'gettext_id' => 'summary mode'},
+		   'notice' => {'gettext_id' => 'notice mode'},
+		   'txt' => {'gettext_id' => 'text only mode'},
+		   'html'=> {'gettext_id' => 'html only mode'},
+		   'urlize' => {'gettext_id' => 'urlize mode'},
+		   'nomail' => {'gettext_id' => 'no mail (usefull for vacations)'},
+		   'not_me' => {'gettext_id' => 'you do not receive your own posts'}
+		   );
 
 ## Cookie expiration periods with corresponding entry in NLS
-%cookie_period = (0     => 1,
-		  10    => 2,
-		  30    => 3, 
-		  60    => 4,
-		  360   => 5,
-		  1440  => 6, 
-		  43200 => 7);
+%cookie_period = (0     => {'gettext_id' => "session"},
+		  10    => {'gettext_id' => "10 minutes"},
+		  30    => {'gettext_id' => "30 minutes"}, 
+		  60    => {'gettext_id' => "1 hour"},
+		  360   => {'gettext_id' => "6 hours"},
+		  1440  => {'gettext_id' => "1 day"}, 
+		  43200 => {'gettext_id' => "1 week"});
 
-%visibility_mode = ('noconceal' => 'public',
-		    'conceal' => 'conceal');
+%visibility_mode = ('noconceal' => {'gettext_id' => "listed in the list review page"},
+		    'conceal' => {'gettext_id' => "concealed"}
+		    );
 
 ## Filenames with corresponding entry in NLS set 15
-%filenames = ('welcome.tpl' => 1,
-	      'bye.tpl' => 2,
-	      'removed.tpl'=> 3,
-	      'message.footer' => 4,
-	      'message.header' => 5,
-	      'remind.tpl' => 6,
-	      'reject.tpl' => 7,
-	      'invite.tpl' => 8,
-	      'helpfile.tpl' => 9,
-	      'lists.tpl' => 10,
-	      'global_remind.tpl' => 11,
-	      'summary.tpl' => 12,
-	      'info' => 13,
-	      'homepage' => 14,
-	      'create_list_request.tpl' => 15,
-	      'list_created.tpl' => 16,
-	      'your_infected_msg.tpl' => 17,
-	      'list_aliases.tpl' => 18
+%filenames = ('welcome.tt2'             => {'gettext_id' => "welcome message"},
+	      'bye.tt2'                 => {'gettext_id' => "unsubscription message"},
+	      'removed.tt2'             => {'gettext_id' => "deletion message"},
+	      'message.footer'          => {'gettext_id' => "message footer"},
+	      'message.header'          => {'gettext_id' => "message header"},
+	      'remind.tt2'              => {'gettext_id' => "remind message"},
+	      'reject.tt2'              => {'gettext_id' => "editor rejection message"},
+	      'invite.tt2'              => {'gettext_id' => "subscribing invitation message"},
+	      'helpfile.tt2'            => {'gettext_id' => "help file"},
+	      'lists.tt2'               => {'gettext_id' => "directory of lists"},
+	      'global_remind.tt2'       => {'gettext_id' => "global remind message"},
+	      'summary.tt2'             => {'gettext_id' => "summary message"},
+	      'info'                    => {'gettext_id' => "list description"},
+	      'homepage'                => {'gettext_id' => "list homepage"},
+	      'create_list_request.tt2' => {'gettext_id' => "list creation request message"},
+	      'list_created.tt2'        => {'gettext_id' => "list creation notification message"},
+	      'your_infected_msg.tt2'   => {'gettext_id' => "virus infection message"},
+	      'list_aliases.tt2'        => {'gettext_id' => "list aliases template"}
 	      );
 
 ## Defined in RFC 1893
@@ -193,7 +193,7 @@ sub load_config {
     }
 
     if ($conf->{'bounce_path'} && (! -d $conf->{'bounce_path'})) {
-	&Log::do_log('err',"No bounces directory: %s", $conf->{'bounce_path'});
+	&Log::do_log('err',"Missing directory '%s' (defined by 'bounce_path' parameter)", $conf->{'bounce_path'});
     }
 
     if ($conf->{'mhonarc'} && (! -x $conf->{'mhonarc'})) {
@@ -216,7 +216,7 @@ sub load_mime_types {
     my $types = {};
 
     @localisation = ('/etc/mime.types', '/usr/local/apache/conf/mime.types',
-		     '/etc/httpd/conf/mime.types','mime.types');
+		     '/etc/httpd/conf/mime.types',$Conf{'etc'}.'/mime.types');
 
     foreach my $loc (@localisation) {
 	next unless (-r $loc);
@@ -258,18 +258,18 @@ sub get_email_from_cookie {
     my ($email, $auth) ;
 
     unless ($secret) {
-	&main::message('error in sympa configuration');
+	&report::reject_report_web('intern','cookie_error',{},'','','',$robot);
 	&Log::do_log('info', 'parameter cookie undefine, authentication failure');
     }
 
     unless ($ENV{'HTTP_COOKIE'}) {
-	&main::message('error in sympa missing cookie');
+	&report::reject_report_web('intern','cookie_error_env',{'env'=> ENV{HTTP_COOKIE}},'get_email_from_cookie','','',$robot);
 	&Log::do_log('info', ' ENV{HTTP_COOKIE} undefined, authentication failure');
     }
 
     ($email, $auth) = &cookielib::check_cookie ($ENV{'HTTP_COOKIE'}, $secret);
     unless ( $email) {
-	&main::message('auth failed');
+	&report::reject_report_web('user','auth_failed',{},'');
 	&Log::do_log('info', 'get_email_from_cookie: auth failed for user %s', $email);
 	return undef;
     }    
@@ -350,7 +350,7 @@ sub init_passwd {
 	    unless ( &List::update_user_db($email,
 					   {'password' => $passwd,
 					    'lang' => $user->{'lang'} || $data->{'lang'}} )) {
-		&main::message('update_failed');
+		&report::reject_report_web('intern','update_user_db_failed',{'user'=>$email},'','',$email,$robot);
 		&Log::do_log('info','init_passwd: update failed');
 		return undef;
 	    }
@@ -361,7 +361,7 @@ sub init_passwd {
 				     'password' => $passwd,
 				     'lang' => $data->{'lang'},
 				     'gecos' => $data->{'gecos'}})) {
-	    &main::message('add_failed');
+	    &report::reject_report_web('intern','add_user_db_failed',{'user'=>$email},'','',$email,$robot);
 	    &Log::do_log('info','init_passwd: add failed');
 	    return undef;
 	}
@@ -375,7 +375,8 @@ sub get_my_url {
 		 
     my $return_url;
     
-    if ($ENV{SSL_PROTOCOL}) {
+    ## Mod_ssl sets SSL_PROTOCOL ; apache-ssl sets SSL_PROTOCOL_VERSION
+    if ($ENV{SSL_PROTOCOL} || $ENV{SSL_PROTOCOL_VERSION}) {
 	$return_url = 'https';
     }else{
 	$return_url = 'http';	
